# -*- coding: utf-8 -*-
# Copyright: (c) 2025 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

import pytest

from ansible.module_utils.basic import AnsibleModule


@pytest.mark.parametrize('value, isbits, expected', [
    ("4KB", False, 4096),
    ("4KB", None, 4096),
    ("4Kb", True, 4096),
])
def test_validator_function(value: str, isbits: bool | None, expected: int) -> None:
    assert AnsibleModule.human_to_bytes(value, isbits=isbits) == expected


@pytest.mark.parametrize('value, expected', [
    ("4KB", 4096),
])
def test_validator_function_default_isbits(value: str, expected: int) -> None:
    assert AnsibleModule.human_to_bytes(value) == expected


@pytest.mark.parametrize('value, isbits', [
    ("4Kb", False),
    ("4KB", True),
])
def test_validator_functions(value: str, isbits: bool) -> None:
    with pytest.raises(ValueError):
        AnsibleModule.human_to_bytes(value, isbits=isbits)



#!/usr/bin/python
from __future__ import annotations


DOCUMENTATION = """
---
module: test_docs_returns
short_description: Test module
description:
    - Test module
author:
    - Ansible Core Team
"""

EXAMPLES = """
"""

RETURN = """
z_last:
    description: A last result.
    type: str
    returned: success

m_middle:
    description:
        - This should be in the middle.
        - Has some more data
    type: dict
    returned: success and 1st of month
    contains:
        suboption:
            description: A suboption.
            type: str
            choices: [ARF, BARN, c_without_capital_first_letter]

a_first:
    description: A first result.
    type: str
    returned: success
"""


from ansible.module_utils.basic import AnsibleModule


def main():
    module = AnsibleModule(
        argument_spec=dict(),
    )

    module.exit_json()


if __name__ == '__main__':
    main()



from __future__ import annotations

import importlib.abc
import importlib.util

import ansible
import pathlib
import pytest

from ansible.module_utils._internal import _messages
from ansible.module_utils._internal import _deprecator


class FakePathLoader(importlib.abc.SourceLoader):
    """A test loader that can fake out the code/frame paths to simulate callers of various types without relying on actual files on disk."""
    def get_filename(self, fullname):
        if fullname.startswith('ansible.'):
            basepath = pathlib.Path(ansible.__file__).parent.parent
        else:
            basepath = '/x/y'

        return f'{basepath}/{fullname.replace(".", "/")}'

    def get_data(self, path):
        return b'''
from ansible.module_utils._internal import _deprecator

def do_stuff():
    return _deprecator.get_caller_plugin_info()
'''

    def exec_module(self, module):
        return super().exec_module(module)


@pytest.mark.parametrize("python_fq_name,expected_plugin_info", (
    # legacy module callers
    ('ansible.legacy.blah', _messages.PluginInfo(resolved_name='ansible.legacy.blah', type=_messages.PluginType.MODULE)),
    # core callers
    ('ansible.modules.ping', _messages.PluginInfo(resolved_name='ansible.builtin.ping', type=_messages.PluginType.MODULE)),
    ('ansible.plugins.filter.core', _deprecator.ANSIBLE_CORE_DEPRECATOR),
    ('ansible.plugins.test.core', _deprecator.ANSIBLE_CORE_DEPRECATOR),
    ('ansible.nonplugin_something', _deprecator.ANSIBLE_CORE_DEPRECATOR),
    # collections plugin callers
    ('ansible_collections.foo.bar.plugins.modules.module_thing', _messages.PluginInfo(resolved_name='foo.bar.module_thing', type=_messages.PluginType.MODULE)),
    ('ansible_collections.foo.bar.plugins.filter.somefilter', _messages.PluginInfo(resolved_name='foo.bar', type=None)),
    ('ansible_collections.foo.bar.plugins.test.sometest', _messages.PluginInfo(resolved_name='foo.bar', type=None)),
    # indeterminate callers (e.g. collection module_utils- must specify since they might be calling on behalf of another
    ('ansible_collections.foo.bar.plugins.module_utils.something', _deprecator.INDETERMINATE_DEPRECATOR),
    # other callers
    ('something.else', None),
    ('ansible_collections.foo.bar.nonplugin_something', None),
))
def test_get_caller_plugin_info(python_fq_name: str, expected_plugin_info: _messages.PluginInfo):
    """Validates the expected `PluginInfo` values received from various types of core/non-core/collection callers."""
    # invoke a standalone fake loader that generates a Python module with the specified FQ python name (converted to a corresponding __file__ entry) that
    # pretends as if it called `get_caller_plugin_info()` and returns its result
    loader = FakePathLoader()
    spec = importlib.util.spec_from_loader(name=python_fq_name, loader=loader)
    mod = importlib.util.module_from_spec(spec)

    loader.exec_module(mod)

    pi: _messages.PluginInfo = mod.do_stuff()

    assert pi == expected_plugin_info



from __future__ import annotations

import sys

prompts = sys.argv[1:] or ['foo']

# UTF8 encoded bytes
# to ensure pexpect doesn't have any encoding errors
data = 'line one 汉语\nline two\nline three\nline four\n'.encode()

sys.stdout.buffer.write(data)
print()

for prompt in prompts:
    user_input = input(prompt)
    print(user_input)



"""Backwards compatibility types, which will be deprecated a future release. Do not use these in new code."""

from __future__ import annotations as _annotations

import typing as _t

from ansible.module_utils._internal import _datatag
from ansible.module_utils.common.text import converters as _converters
from ansible.parsing import vault as _vault

_UNSET = _t.cast(_t.Any, object())


class _AnsibleMapping(dict):
    """Backwards compatibility type."""

    def __new__(cls, value=_UNSET, /, **kwargs):
        if value is _UNSET:
            return dict(**kwargs)

        return _datatag.AnsibleTagHelper.tag_copy(value, dict(value, **kwargs))


class _AnsibleUnicode(str):
    """Backwards compatibility type."""

    def __new__(cls, object=_UNSET, **kwargs):
        if object is _UNSET:
            return str(**kwargs)

        return _datatag.AnsibleTagHelper.tag_copy(object, str(object, **kwargs))


class _AnsibleSequence(list):
    """Backwards compatibility type."""

    def __new__(cls, value=_UNSET, /):
        if value is _UNSET:
            return list()

        return _datatag.AnsibleTagHelper.tag_copy(value, list(value))


class _AnsibleVaultEncryptedUnicode:
    """Backwards compatibility type."""

    def __new__(cls, ciphertext: str | bytes):
        encrypted_string = _vault.EncryptedString(ciphertext=_converters.to_text(_datatag.AnsibleTagHelper.untag(ciphertext)))

        return _datatag.AnsibleTagHelper.tag_copy(ciphertext, encrypted_string)


def __getattr__(name: str) -> _t.Any:
    """Inject import-time deprecation warnings."""
    if (value := globals().get(f'_{name}', None)) and name.startswith('Ansible'):
        # deprecated: description='enable deprecation of everything in this module', core_version='2.23'
        # from ansible.utils.display import Display
        #
        # Display().deprecated(
        #     msg=f"Importing {name!r} is deprecated.",
        #     help_text="Instances of this type cannot be created and will not be encountered.",
        #     version="2.27",
        # )

        return value

    raise AttributeError(f'module {__name__!r} has no attribute {name!r}')



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2015, Jon Hawkesworth (@jhawkesworth) <figs@unity.demon.co.uk>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['stableinterface'],
                    'supported_by': 'core'}

DOCUMENTATION = r"""
---
module: win_file
version_added: "1.9.2"
short_description: Creates, touches or removes files or directories
description:
     - Creates (empty) files, updates file modification stamps of existing files,
       and can create or remove directories.
     - Unlike M(file), does not modify ownership, permissions or manipulate links.
     - For non-Windows targets, use the M(file) module instead.
options:
  path:
    description:
      - Path to the file being managed.
    required: yes
    type: path
    aliases: [ dest, name ]
  state:
    description:
      - If C(directory), all immediate subdirectories will be created if they
        do not exist.
      - If C(file), the file will NOT be created if it does not exist, see the M(copy)
        or M(template) module if you want that behavior.
      - If C(absent), directories will be recursively deleted, and files will be removed.
      - If C(touch), an empty file will be created if the C(path) does not
        exist, while an existing file or directory will receive updated file access and
        modification times (similar to the way C(touch) works from the command line).
    type: str
    choices: [ absent, directory, file, touch ]
seealso:
- module: file
- module: win_acl
- module: win_acl_inheritance
- module: win_owner
- module: win_stat
author:
- Jon Hawkesworth (@jhawkesworth)
"""

EXAMPLES = r"""
- name: Touch a file (creates if not present, updates modification time if present)
  win_file:
    path: C:\Temp\foo.conf
    state: touch

- name: Remove a file, if present
  win_file:
    path: C:\Temp\foo.conf
    state: absent

- name: Create directory structure
  win_file:
    path: C:\Temp\folder\subfolder
    state: directory

- name: Remove directory structure
  win_file:
    path: C:\Temp
    state: absent
"""



#!/usr/bin/python
#
# Copyright 2018 Red Hat | Ansible
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

import resource
import subprocess

from ansible.module_utils.basic import AnsibleModule


def main():
    module = AnsibleModule(
        argument_spec=dict()
    )

    rlimit_nofile = resource.getrlimit(resource.RLIMIT_NOFILE)

    try:
        maxfd = subprocess.MAXFD
    except AttributeError:
        maxfd = -1

    module.exit_json(rlimit_nofile=rlimit_nofile, maxfd=maxfd, infinity=resource.RLIM_INFINITY)


if __name__ == '__main__':
    main()



# (c) 2021 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations


DOCUMENTATION = """
    name: casting
    author: Ansible Core Team
    version_added: histerical
    short_description: returns what you gave it
    description:
      - this is mostly a noop
    options:
        _terms:
            description: stuff to pass through
        test_list:
            description: does nothihng, just for testing values
            type: list
        test_int:
            description: does nothihng, just to test casting
            type: int
        test_bool:
            description: does nothihng, just to test casting
            type: bool
        test_str:
            description: does nothihng, just to test casting
            type: str
"""

EXAMPLES = """
- name: like some other plugins, this is mostly useless
  debug: msg={{ q('casting', [1,2,3])}}
"""

RETURN = """
  _list:
    description: basically the same as you fed in
    type: list
    elements: raw
"""

from ansible.plugins.lookup import LookupBase


class LookupModule(LookupBase):

    def run(self, terms, variables=None, **kwargs):

        self.set_options(var_options=variables, direct=kwargs)

        for cast in (list, int, bool, str):
            option = 'test_%s' % str(cast).replace("<class '", '').replace("'>", '')
            value = self.get_option(option)
            if value is None or isinstance(value, cast):
                continue
            raise Exception('%s is not a %s: got %s/%s' % (option, cast, type(value), value))

        return terms



from __future__ import annotations

import copy
import dataclasses
import functools
import types
import typing as t

from jinja2.environment import TemplateModule

from ansible.module_utils._internal._datatag import (
    AnsibleTagHelper,
    AnsibleTaggedObject,
    _AnsibleTaggedDict,
    _AnsibleTaggedList,
    _AnsibleTaggedTuple,
    _NO_INSTANCE_STORAGE,
    _try_get_internal_tags_mapping,
)

from ansible.utils.sentinel import Sentinel
from ansible.errors import AnsibleVariableTypeError
from ansible._internal._errors._handler import Skippable
from ansible.vars.hostvars import HostVarsVars, HostVars

from ._access import AnsibleAccessContext
from ._jinja_common import Marker, _TemplateConfig
from ._utils import TemplateContext, PASS_THROUGH_SCALAR_VAR_TYPES, LazyOptions

if t.TYPE_CHECKING:
    from ._engine import TemplateEngine

_KNOWN_TYPES: t.Final[set[type]] = (
    {
        HostVars,  # example: hostvars
        HostVarsVars,  # example: hostvars.localhost | select
        type,  # example: range(20) | list  # triggered on retrieval of `range` type from globals
        range,  # example: range(20) | list  # triggered when returning a `range` instance from a call
        types.FunctionType,  # example: undef() | default("blah")
        types.MethodType,  # example: ansible_facts.get | type_debug
        functools.partial,
        type(''.startswith),  # example: inventory_hostname.upper | type_debug  # using `startswith` to resolve `builtin_function_or_method`
        TemplateModule,  # example: '{% import "importme.j2" as im %}{{ im | type_debug }}'
    }
    | set(PASS_THROUGH_SCALAR_VAR_TYPES)
    | set(Marker._concrete_subclasses)
)
"""
These types are known to the templating system.
In addition to the statically defined types, additional types will be added at runtime.
When enabled in config, this set will be used to determine if an encountered type should trigger a warning or error.
"""


def register_known_types(*args: type) -> None:
    """Register a type with the template engine so it will not trigger warnings or errors when encountered."""
    _KNOWN_TYPES.update(args)


class UnsupportedConstructionMethodError(RuntimeError):
    """Error raised when attempting to construct a lazy container with unsupported arguments."""

    def __init__(self):
        super().__init__("Direct construction of lazy containers is not supported.")


@t.final
@dataclasses.dataclass(frozen=True, slots=True)
class _LazyValue:
    """Wrapper around values to indicate lazy behavior has not yet been applied."""

    value: t.Any


@t.final
@dataclasses.dataclass(frozen=True, kw_only=True, slots=True)
class _LazyValueSource:
    """Intermediate value source for lazy-eligible collection copy operations."""

    source: t.Iterable
    templar: TemplateEngine
    lazy_options: LazyOptions


@t.final
class _NoKeySentinel(Sentinel):
    """Sentinel used to indicate a requested key was not found."""


# There are several operations performed by lazy containers, with some variation between types.
#
# Columns: D=dict, L=list, T=tuple
#   Cells: l=lazy (upon access), n=non-lazy (__init__/__new__)
#
# D  L  T  Feature      Description
# -  -  -  -----------  ---------------------------------------------------------------
# l  l  n  propagation  when container items which are containers become lazy instances
# l  l  n  transform    when transforms are applied to container items
# l  l  n  templating   when templating is performed on container items
# l  l  l  access       when access calls are performed on container items


class _AnsibleLazyTemplateMixin:
    __slots__ = _NO_INSTANCE_STORAGE

    _dispatch_types: t.ClassVar[dict[type, type[_AnsibleLazyTemplateMixin]]] = {}  # populated by __init_subclass__
    _container_types: t.ClassVar[set[type]] = set()  # populated by __init_subclass__

    _native_type: t.ClassVar[type]  # from AnsibleTaggedObject

    _SLOTS: t.Final = (
        '_templar',
        '_lazy_options',
    )

    _templar: TemplateEngine
    _lazy_options: LazyOptions

    def __init_subclass__(cls, **kwargs) -> None:
        tagged_type = cls.__mro__[1]
        native_type = tagged_type.__mro__[1]

        for check_type in (tagged_type, native_type):
            if conflicting_type := cls._dispatch_types.get(check_type):
                raise TypeError(f"Lazy mixin {cls.__name__!r} type {check_type.__name__!r} conflicts with {conflicting_type.__name__!r}.")

        cls._dispatch_types[native_type] = cls
        cls._dispatch_types[tagged_type] = cls
        cls._container_types.add(native_type)
        cls._empty_tags_as_native = False  # never revert to the native type when no tags remain

        register_known_types(cls)

    def __init__(self, contents: t.Iterable | _LazyValueSource) -> None:
        if isinstance(contents, _LazyValueSource):
            self._templar = contents.templar
            self._lazy_options = contents.lazy_options
        elif isinstance(contents, _AnsibleLazyTemplateMixin):
            self._templar = contents._templar
            self._lazy_options = contents._lazy_options
        else:
            raise UnsupportedConstructionMethodError()

    def __reduce_ex__(self, protocol):
        raise NotImplementedError("Pickling of Ansible lazy objects is not permitted.")

    @staticmethod
    def _try_create(item: t.Any, lazy_options: LazyOptions = LazyOptions.DEFAULT) -> t.Any:
        """
        If `item` is a container type which supports lazy access and/or templating, return a lazy wrapped version -- otherwise return it as-is.
        When returning as-is, a warning or error may be generated for unknown types.
        The `lazy_options.skip_templates` argument should be set to `True` when `item` is sourced from a plugin instead of Ansible variable storage.
        This provides backwards compatibility and reduces lazy overhead, as plugins do not normally introduce templates.
        If a plugin needs to introduce templates, the plugin is responsible for invoking the templar and returning the result.
        """
        item_type = type(item)

        # Try to use exact type match first to determine which wrapper (if any) to apply; isinstance checks
        # are extremely expensive, so try to avoid them for our commonly-supported types.
        if (dispatcher := _AnsibleLazyTemplateMixin._dispatch_types.get(item_type)) is not None:
            # Create a generator that yields the elements of `item` wrapped in a `_LazyValue` wrapper.
            # The wrapper is used to signal to the lazy container that the value must be processed before being returned.
            # Values added to the lazy container later through other means will be returned as-is, without any special processing.
            lazy_values = dispatcher._lazy_values(item, lazy_options)
            tags_mapping = _try_get_internal_tags_mapping(item)
            value = t.cast(AnsibleTaggedObject, dispatcher)._instance_factory(lazy_values, tags_mapping)

            return value

        with Skippable, _TemplateConfig.unknown_type_encountered_handler.handle(AnsibleVariableTypeError, skip_on_ignore=True):
            if item_type not in _KNOWN_TYPES:
                raise AnsibleVariableTypeError(
                    message=f"Encountered unknown type {item_type.__name__!r} during template operation.",
                    help_text="Use supported types to avoid unexpected behavior.",
                    obj=TemplateContext.current().template_value,
                )

        return item

    def _is_not_lazy_combine_candidate(self, other: object) -> bool:
        """Returns `True` if `other` cannot be lazily combined with the current instance due to differing templar/options, otherwise returns `False`."""
        return isinstance(other, _AnsibleLazyTemplateMixin) and (self._templar is not other._templar or self._lazy_options != other._lazy_options)

    def _non_lazy_copy(self) -> t.Collection:
        """
        Return a non-lazy copy of this collection.
        Any remaining lazy wrapped values will be unwrapped without further processing.
        Tags on this instance will be preserved on the returned copy.
        """
        raise NotImplementedError()  # pragma: nocover

    @staticmethod
    def _lazy_values(values: t.Any, lazy_options: LazyOptions) -> _LazyValueSource:
        """
        Return an iterable that wraps each of the given elements in a lazy wrapper.
        Only elements wrapped this way will receive lazy processing when retrieved from the collection.
        """
        # DTFIX-FUTURE: check relative performance of method-local vs stored generator expressions on implementations of this method
        raise NotImplementedError()  # pragma: nocover

    def _proxy_or_render_lazy_value(self, key: t.Any, value: t.Any) -> t.Any:
        """
        Ensure that the value is lazy-proxied or rendered, and if a key is provided, replace the original value with the result.
        """
        if type(value) is not _LazyValue:  # pylint: disable=unidiomatic-typecheck
            if self._lazy_options.access:
                AnsibleAccessContext.current().access(value)

            return value

        original_value = value.value

        if self._lazy_options.access:
            AnsibleAccessContext.current().access(original_value)

        new_value = self._templar.template(original_value, lazy_options=self._lazy_options)

        if new_value is not original_value and self._lazy_options.access:
            AnsibleAccessContext.current().access(new_value)

        if key is not _NoKeySentinel:
            self._native_type.__setitem__(self, key, new_value)  # type: ignore  # pylint: disable=unnecessary-dunder-call

        return new_value


@t.final  # consumers of lazy collections rely heavily on the concrete types being final
class _AnsibleLazyTemplateDict(_AnsibleTaggedDict, _AnsibleLazyTemplateMixin):
    __slots__ = _AnsibleLazyTemplateMixin._SLOTS

    def __init__(self, contents: t.Iterable | _LazyValueSource, /, **kwargs) -> None:
        if isinstance(contents, _AnsibleLazyTemplateDict):
            super().__init__(dict.items(contents), **kwargs)
        elif isinstance(contents, _LazyValueSource):
            super().__init__(contents.source, **kwargs)
        else:
            raise UnsupportedConstructionMethodError()

        _AnsibleLazyTemplateMixin.__init__(self, contents)

    def get(self, key: t.Any, default: t.Any = None) -> t.Any:
        if (value := super().get(key, _NoKeySentinel)) is _NoKeySentinel:
            return default

        return self._proxy_or_render_lazy_value(key, value)

    def __getitem__(self, key: t.Any, /) -> t.Any:
        return self._proxy_or_render_lazy_value(key, super().__getitem__(key))

    def __str__(self):
        return str(self.copy()._native_copy())  # inefficient, but avoids mutating the current instance (to make debugging practical)

    def __repr__(self):
        return repr(self.copy()._native_copy())  # inefficient, but avoids mutating the current instance (to make debugging practical)

    def __iter__(self):
        # We're using the base implementation, but must override `__iter__` to skip `dict` fast-path copy, which would bypass lazy behavior.
        # See: https://github.com/python/cpython/blob/ffcc450a9b8b6927549b501eff7ac14abc238448/Objects/dictobject.c#L3861-L3864
        return super().__iter__()

    def setdefault(self, key, default=None, /) -> t.Any:
        if (value := self.get(key, _NoKeySentinel)) is not _NoKeySentinel:
            return value

        super().__setitem__(key, default)

        return default

    def items(self):
        for key, value in super().items():
            yield key, self._proxy_or_render_lazy_value(key, value)

    def values(self):
        for _key, value in self.items():
            yield value

    def pop(self, key, default=_NoKeySentinel, /) -> t.Any:
        if (value := super().get(key, _NoKeySentinel)) is _NoKeySentinel:
            if default is _NoKeySentinel:
                raise KeyError(key)

            return default

        value = self._proxy_or_render_lazy_value(_NoKeySentinel, value)

        del self[key]

        return value

    def popitem(self) -> t.Any:
        try:
            key = next(reversed(self))
        except StopIteration:
            raise KeyError("popitem(): dictionary is empty")

        value = self._proxy_or_render_lazy_value(_NoKeySentinel, self[key])

        del self[key]

        return key, value

    def _native_copy(self) -> dict:
        return dict(self.items())

    @staticmethod
    def _item_source(value: dict) -> dict | _LazyValueSource:
        if isinstance(value, _AnsibleLazyTemplateDict):
            return _LazyValueSource(source=dict.items(value), templar=value._templar, lazy_options=value._lazy_options)

        return value

    def _yield_non_lazy_dict_items(self) -> t.Iterator[tuple[str, t.Any]]:
        """
        Delegate to the base collection items iterator to yield the raw contents.
        As of Python 3.13, generator functions are significantly faster than inline generator expressions.
        """
        for k, v in dict.items(self):
            yield k, v.value if type(v) is _LazyValue else v  # pylint: disable=unidiomatic-typecheck

    def _non_lazy_copy(self) -> dict:
        return AnsibleTagHelper.tag_copy(self, self._yield_non_lazy_dict_items(), value_type=dict)

    @staticmethod
    def _lazy_values(values: dict, lazy_options: LazyOptions) -> _LazyValueSource:
        return _LazyValueSource(source=((k, _LazyValue(v)) for k, v in values.items()), templar=TemplateContext.current().templar, lazy_options=lazy_options)

    @staticmethod
    def _proxy_or_render_other(other: t.Any | None) -> None:
        """Call `_proxy_or_render_lazy_values` if `other` is a lazy dict. Used internally by comparison methods."""
        if type(other) is _AnsibleLazyTemplateDict:  # pylint: disable=unidiomatic-typecheck
            other._proxy_or_render_lazy_values()

    def _proxy_or_render_lazy_values(self) -> None:
        """Ensure all `_LazyValue` wrapped values have been processed."""
        for _unused in self.values():
            pass

    def __eq__(self, other):
        self._proxy_or_render_lazy_values()
        self._proxy_or_render_other(other)
        return super().__eq__(other)

    def __ne__(self, other):
        self._proxy_or_render_lazy_values()
        self._proxy_or_render_other(other)
        return super().__ne__(other)

    def __or__(self, other):
        # DTFIX-FUTURE: support preservation of laziness when possible like we do for list
        # Both sides end up going through _proxy_or_render_lazy_value, so there's no Templar preservation needed.
        # In the future this could be made more lazy when both Templar instances are the same, or if per-value Templar tracking was used.
        return super().__or__(other)

    def __ror__(self, other):
        # DTFIX-FUTURE: support preservation of laziness when possible like we do for list
        # Both sides end up going through _proxy_or_render_lazy_value, so there's no Templar preservation needed.
        # In the future this could be made more lazy when both Templar instances are the same, or if per-value Templar tracking was used.
        return super().__ror__(other)

    def __deepcopy__(self, memo):
        return _AnsibleLazyTemplateDict(
            _LazyValueSource(
                source=((copy.deepcopy(k), copy.deepcopy(v)) for k, v in super().items()),
                templar=copy.deepcopy(self._templar),
                lazy_options=copy.deepcopy(self._lazy_options),
            )
        )


@t.final  # consumers of lazy collections rely heavily on the concrete types being final
class _AnsibleLazyTemplateList(_AnsibleTaggedList, _AnsibleLazyTemplateMixin):
    __slots__ = _AnsibleLazyTemplateMixin._SLOTS

    def __init__(self, contents: t.Iterable | _LazyValueSource, /) -> None:
        if isinstance(contents, _AnsibleLazyTemplateList):
            super().__init__(list.__iter__(contents))
        elif isinstance(contents, _LazyValueSource):
            super().__init__(contents.source)
        else:
            raise UnsupportedConstructionMethodError()

        _AnsibleLazyTemplateMixin.__init__(self, contents)

    def __getitem__(self, key: t.SupportsIndex | slice, /) -> t.Any:
        if type(key) is slice:  # pylint: disable=unidiomatic-typecheck
            return _AnsibleLazyTemplateList(_LazyValueSource(source=super().__getitem__(key), templar=self._templar, lazy_options=self._lazy_options))

        return self._proxy_or_render_lazy_value(key, super().__getitem__(key))

    def __iter__(self):
        for key, value in enumerate(super().__iter__()):
            yield self._proxy_or_render_lazy_value(key, value)

    def pop(self, idx: t.SupportsIndex = -1, /) -> t.Any:
        if not self:
            raise IndexError('pop from empty list')

        try:
            value = self[idx]
        except IndexError:
            raise IndexError('pop index out of range')

        value = self._proxy_or_render_lazy_value(_NoKeySentinel, value)

        del self[idx]

        return value

    def __str__(self):
        return str(self.copy()._native_copy())  # inefficient, but avoids mutating the current instance (to make debugging practical)

    def __repr__(self):
        return repr(self.copy()._native_copy())  # inefficient, but avoids mutating the current instance (to make debugging practical)

    @staticmethod
    def _item_source(value: list) -> list | _LazyValueSource:
        if isinstance(value, _AnsibleLazyTemplateList):
            return _LazyValueSource(source=list.__iter__(value), templar=value._templar, lazy_options=value._lazy_options)

        return value

    def _yield_non_lazy_list_items(self):
        """
        Delegate to the base collection iterator to yield the raw contents.
        As of Python 3.13, generator functions are significantly faster than inline generator expressions.
        """
        for v in list.__iter__(self):
            yield v.value if type(v) is _LazyValue else v  # pylint: disable=unidiomatic-typecheck

    def _non_lazy_copy(self) -> list:
        return AnsibleTagHelper.tag_copy(self, self._yield_non_lazy_list_items(), value_type=list)

    @staticmethod
    def _lazy_values(values: list, lazy_options: LazyOptions) -> _LazyValueSource:
        return _LazyValueSource(source=(_LazyValue(v) for v in values), templar=TemplateContext.current().templar, lazy_options=lazy_options)

    @staticmethod
    def _proxy_or_render_other(other: t.Any | None) -> None:
        """Call `_proxy_or_render_lazy_values` if `other` is a lazy list. Used internally by comparison methods."""
        if type(other) is _AnsibleLazyTemplateList:  # pylint: disable=unidiomatic-typecheck
            other._proxy_or_render_lazy_values()

    def _proxy_or_render_lazy_values(self) -> None:
        """Ensure all `_LazyValue` wrapped values have been processed."""
        for _unused in self:
            pass

    def __eq__(self, other):
        self._proxy_or_render_lazy_values()
        self._proxy_or_render_other(other)
        return super().__eq__(other)

    def __ne__(self, other):
        self._proxy_or_render_lazy_values()
        self._proxy_or_render_other(other)
        return super().__ne__(other)

    def __gt__(self, other):
        self._proxy_or_render_lazy_values()
        self._proxy_or_render_other(other)
        return super().__gt__(other)

    def __ge__(self, other):
        self._proxy_or_render_lazy_values()
        self._proxy_or_render_other(other)
        return super().__ge__(other)

    def __lt__(self, other):
        self._proxy_or_render_lazy_values()
        self._proxy_or_render_other(other)
        return super().__lt__(other)

    def __le__(self, other):
        self._proxy_or_render_lazy_values()
        self._proxy_or_render_other(other)
        return super().__le__(other)

    def __contains__(self, item):
        self._proxy_or_render_lazy_values()
        return super().__contains__(item)

    def __reversed__(self):
        for idx in range(self.__len__() - 1, -1, -1):
            yield self[idx]

    def __add__(self, other):
        if self._is_not_lazy_combine_candidate(other):
            # When other is lazy with a different templar/options, it cannot be lazily combined with self and a plain list must be returned.
            # If other is a list, de-lazify both, otherwise just let the operation fail.

            if isinstance(other, _AnsibleLazyTemplateList):
                self._proxy_or_render_lazy_values()
                other._proxy_or_render_lazy_values()

            return super().__add__(other)

        # For all other cases, the new list inherits our templar and all values stay lazy.
        # We use list.__add__ to avoid implementing all its error behavior.
        return _AnsibleLazyTemplateList(_LazyValueSource(source=super().__add__(other), templar=self._templar, lazy_options=self._lazy_options))

    def __radd__(self, other):
        if not (other_add := getattr(other, '__add__', None)):
            raise TypeError(f'unsupported operand type(s) for +: {type(other).__name__!r} and {type(self).__name__!r}') from None

        return _AnsibleLazyTemplateList(_LazyValueSource(source=other_add(self), templar=self._templar, lazy_options=self._lazy_options))

    def __mul__(self, other):
        return _AnsibleLazyTemplateList(_LazyValueSource(source=super().__mul__(other), templar=self._templar, lazy_options=self._lazy_options))

    def __rmul__(self, other):
        return _AnsibleLazyTemplateList(_LazyValueSource(source=super().__rmul__(other), templar=self._templar, lazy_options=self._lazy_options))

    def index(self, *args, **kwargs) -> int:
        self._proxy_or_render_lazy_values()
        return super().index(*args, **kwargs)

    def remove(self, *args, **kwargs) -> None:
        self._proxy_or_render_lazy_values()
        super().remove(*args, **kwargs)

    def sort(self, *args, **kwargs) -> None:
        self._proxy_or_render_lazy_values()
        super().sort(*args, **kwargs)

    def __deepcopy__(self, memo):
        return _AnsibleLazyTemplateList(
            _LazyValueSource(
                source=(copy.deepcopy(v) for v in super().__iter__()),
                templar=copy.deepcopy(self._templar),
                lazy_options=copy.deepcopy(self._lazy_options),
            )
        )


@t.final  # consumers of lazy collections rely heavily on the concrete types being final
class _AnsibleLazyAccessTuple(_AnsibleTaggedTuple, _AnsibleLazyTemplateMixin):
    """
    A tagged tuple subclass that provides only managed access for existing lazy values.

    Since tuples are immutable, they cannot support lazy templating (which would change the tuple's value as templates were resolved).
    When this type is created, each value in the source tuple is lazified:

    * template strings are templated immediately (possibly resulting in lazy containers)
    * non-tuple containers are lazy-wrapped
    * tuples are immediately recursively lazy-wrapped
    * transformations are applied immediately

    The resulting object provides only managed access to its values (e.g., deprecation warnings, tripwires), and propagates to new lazy containers
    created as a results of managed access.
    """

    # DTFIX5: ensure we have tests that explicitly verify this behavior

    # nonempty __slots__ not supported for subtype of 'tuple'

    def __new__(cls, contents: t.Iterable | _LazyValueSource, /) -> t.Self:
        if isinstance(contents, _AnsibleLazyAccessTuple):
            return super().__new__(cls, tuple.__iter__(contents))

        if isinstance(contents, _LazyValueSource):
            return super().__new__(cls, contents.source)

        raise UnsupportedConstructionMethodError()

    def __init__(self, contents: t.Iterable | _LazyValueSource, /) -> None:
        _AnsibleLazyTemplateMixin.__init__(self, contents)

    def __getitem__(self, key: t.SupportsIndex | slice, /) -> t.Any:
        if type(key) is slice:  # pylint: disable=unidiomatic-typecheck
            return _AnsibleLazyAccessTuple(_LazyValueSource(source=super().__getitem__(key), templar=self._templar, lazy_options=self._lazy_options))

        value = super().__getitem__(key)

        if self._lazy_options.access:
            AnsibleAccessContext.current().access(value)

        return value

    @staticmethod
    def _item_source(value: tuple) -> tuple | _LazyValueSource:
        if isinstance(value, _AnsibleLazyAccessTuple):
            return _LazyValueSource(source=tuple.__iter__(value), templar=value._templar, lazy_options=value._lazy_options)

        return value

    @staticmethod
    def _lazy_values(values: t.Any, lazy_options: LazyOptions) -> _LazyValueSource:
        templar = TemplateContext.current().templar

        return _LazyValueSource(source=(templar.template(value, lazy_options=lazy_options) for value in values), templar=templar, lazy_options=lazy_options)

    def _non_lazy_copy(self) -> tuple:
        return AnsibleTagHelper.tag_copy(self, self, value_type=tuple)

    def __deepcopy__(self, memo):
        return _AnsibleLazyAccessTuple(
            _LazyValueSource(
                source=(copy.deepcopy(v) for v in super().__iter__()),
                templar=copy.deepcopy(self._templar),
                lazy_options=copy.deepcopy(self._lazy_options),
            )
        )


def lazify_container(value: t.Any) -> t.Any:
    """
    If the given value is a supported container type, return its lazy version, otherwise return the value as-is.
    This is used to ensure that managed access and templating occur on args and kwargs to a callable, even if they were sourced from Jinja constants.

    Since both variable access and plugin output are already lazified, this mostly affects Jinja constant containers.
    However, plugins that directly invoke other plugins (e.g., `Environment.call_filter`) are another potential source of non-lazy containers.
    In these cases, templating will occur for trusted templates automatically upon access.

    Sets, tuples, and dictionary keys cannot be lazy, since their correct operation requires hashability and equality.
    These properties are mutually exclusive with the following lazy features:

    - managed access on encrypted strings - may raise errors on both operations when decryption fails
    - managed access on markers - must raise errors on both operations
    - templating - mutates values

    That leaves non-raising managed access as the only remaining feature, which is insufficient to warrant lazy support.
    """
    return _AnsibleLazyTemplateMixin._try_create(value)


def lazify_container_args(item: tuple) -> tuple:
    """Return the given args with values converted to lazy containers as needed."""
    return tuple(lazify_container(value) for value in item)


def lazify_container_kwargs(item: dict[str, t.Any]) -> dict[str, t.Any]:
    """Return the given kwargs with values converted to lazy containers as needed."""
    return {key: lazify_container(value) for key, value in item.items()}



from __future__ import annotations

import datetime

import pytest

from ansible.module_utils.compat import datetime as compat_datetime


pytestmark = pytest.mark.usefixtures('capfd')  # capture deprecation warnings


def test_utc():
    assert compat_datetime.UTC.tzname(None) == 'UTC'
    assert compat_datetime.UTC.utcoffset(None) == datetime.timedelta(0)

    assert compat_datetime.UTC.dst(None) is None


def test_utcnow():
    assert compat_datetime.utcnow().tzinfo is compat_datetime.UTC


def test_utcfometimestamp_zero():
    dt = compat_datetime.utcfromtimestamp(0)

    assert dt.tzinfo is compat_datetime.UTC
    assert dt.year == 1970
    assert dt.month == 1
    assert dt.day == 1
    assert dt.hour == 0
    assert dt.minute == 0
    assert dt.second == 0
    assert dt.microsecond == 0



# (c) 2012-2014, Ansible, Inc
# (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

DOCUMENTATION = """
    name: tree
    type: notification
    requirements:
      - invoked in the command line
    short_description: Save host events to files
    version_added: "2.0"
    options:
        directory:
            version_added: '2.11'
            description: directory that will contain the per host JSON files. Also set by the C(--tree) option when using adhoc.
            ini:
                - section: callback_tree
                  key: directory
            env:
                - name: ANSIBLE_CALLBACK_TREE_DIR
            default: "~/.ansible/tree"
            type: path
    description:
        - "This callback is used by the Ansible (adhoc) command line option C(-t|--tree)."
        - This produces a JSON dump of events in a directory, a file for each host, the directory used MUST be passed as a command line option.
"""

import os

from ansible.constants import TREE_DIR
from ansible.executor.task_result import CallbackTaskResult
from ansible.module_utils.common.text.converters import to_bytes
from ansible.plugins.callback import CallbackBase
from ansible.utils.path import makedirs_safe, unfrackpath
from ansible.module_utils._internal import _deprecator


class CallbackModule(CallbackBase):
    """
    This callback puts results into a host specific file in a directory in json format.
    """

    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = 'aggregate'
    CALLBACK_NAME = 'tree'
    CALLBACK_NEEDS_ENABLED = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self._display.deprecated(  # pylint: disable=ansible-deprecated-unnecessary-collection-name
            msg='The tree callback plugin is deprecated.',
            version='2.23',
            deprecator=_deprecator.ANSIBLE_CORE_DEPRECATOR,  # entire plugin being removed; this improves the messaging
        )

    def set_options(self, task_keys=None, var_options=None, direct=None):
        """ override to set self.tree """

        super(CallbackModule, self).set_options(task_keys=task_keys, var_options=var_options, direct=direct)

        if TREE_DIR:
            # TREE_DIR comes from the CLI option --tree, only available for adhoc
            self.tree = unfrackpath(TREE_DIR)
        else:
            self.tree = self.get_option('directory')

    def write_tree_file(self, hostname, buf):
        """ write something into treedir/hostname """

        buf = to_bytes(buf)
        try:
            makedirs_safe(self.tree)
        except OSError as ex:
            self._display.error_as_warning(f"Unable to access or create the configured directory {self.tree!r}.", exception=ex)

        try:
            path = to_bytes(os.path.join(self.tree, hostname))
            with open(path, 'wb+') as fd:
                fd.write(buf)
        except OSError as ex:
            self._display.error_as_warning(f"Unable to write to {hostname!r}'s file.", exception=ex)

    def result_to_tree(self, result: CallbackTaskResult) -> None:
        self.write_tree_file(result.host.get_name(), self._dump_results(result.result))

    def v2_runner_on_ok(self, result: CallbackTaskResult) -> None:
        self.result_to_tree(result)

    def v2_runner_on_failed(self, result: CallbackTaskResult, ignore_errors: bool = False) -> None:
        self.result_to_tree(result)

    def v2_runner_on_unreachable(self, result: CallbackTaskResult) -> None:
        self.result_to_tree(result)



from __future__ import annotations

from ansible._internal._errors._task_timeout import TaskTimeoutError
from ansible.module_utils._internal._datatag._tags import Deprecated


def test_task_timeout_result_contribution() -> None:
    """Validate the result contribution shape."""
    try:
        raise TaskTimeoutError(99)
    except TaskTimeoutError as tte:
        contrib = tte.result_contribution

        assert isinstance(contrib, dict)

        timedout = contrib.get('timedout')

        assert isinstance(timedout, dict)

        frame = timedout.get('frame')

        assert isinstance(frame, str)
        assert Deprecated.is_tagged_on(frame)

        period = timedout.get('period')

        assert period == 99



from __future__ import annotations

import socket

from ansible.module_utils._internal._datatag._tags import Deprecated


def test_getaddrinfo() -> None:
    """Verify that `socket.getaddrinfo` works with a tagged port."""
    # DTFIX5: add additional args and validate output shape (ensure passthru is working)
    socket.getaddrinfo('localhost', Deprecated(msg='').tag(22))



# (c) 2012-2014, Michael DeHaan <michael.dehaan@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

import collections.abc as _c
import enum
import types
import typing as t

from json import JSONDecodeError

from ansible.module_utils.common.text.converters import to_text
from ..module_utils.datatag import native_type_name
from ansible._internal._datatag import _tags
from .._internal._errors import _error_utils
from ansible.module_utils._internal import _text_utils

if t.TYPE_CHECKING:
    from ansible.plugins import loader as _t_loader


class ExitCode(enum.IntEnum):
    SUCCESS = 0  # used by TQM, must be bit-flag safe
    GENERIC_ERROR = 1  # used by TQM, must be bit-flag safe
    HOST_FAILED = 2  # TQM-sourced, must be bit-flag safe
    HOST_UNREACHABLE = 4  # TQM-sourced, must be bit-flag safe
    PARSER_ERROR = 4  # FIXME: CLI-sourced, conflicts with HOST_UNREACHABLE
    INVALID_CLI_OPTION = 5
    UNICODE_ERROR = 6  # obsolete, no longer used
    KEYBOARD_INTERRUPT = 99
    UNKNOWN_ERROR = 250


class AnsibleError(Exception):
    """
    This is the base class for all errors raised from Ansible code,
    and can be instantiated with two optional parameters beyond the
    error message to control whether detailed information is displayed
    when the error occurred while parsing a data file of some kind.

    Usage:

        raise AnsibleError('some message here', obj=obj)

    Where "obj" may be tagged with Origin to provide context for error messages.
    """

    _exit_code = ExitCode.GENERIC_ERROR
    _default_message = ''
    _default_help_text: str | None = None
    _include_cause_message = True
    """
    When `True`, the exception message will be augmented with cause message(s).
    Subclasses doing complex error analysis can disable this to take responsibility for reporting cause messages as needed.
    """

    def __init__(
        self,
        message: str = "",
        obj: t.Any = None,
        show_content: bool = True,
        suppress_extended_error: bool | types.EllipsisType = ...,
        orig_exc: BaseException | None = None,
        help_text: str | None = None,
    ) -> None:
        # DTFIX-FUTURE: these fallback cases mask incorrect use of AnsibleError.message, what should we do?
        if message is None:
            message = ''
        elif not isinstance(message, str):
            message = str(message)

        if self._default_message and message:
            message = _text_utils.concat_message(self._default_message, message)
        elif self._default_message:
            message = self._default_message
        elif not message:
            message = f'Unexpected {type(self).__name__} error.'

        super().__init__(message)

        self._show_content = show_content
        self._message = message
        self._help_text_value = help_text or self._default_help_text
        self.obj = obj

        # deprecated: description='deprecate support for orig_exc, callers should use `raise ... from` only' core_version='2.23'
        # deprecated: description='remove support for orig_exc' core_version='2.27'
        self.orig_exc = orig_exc

        if suppress_extended_error is not ...:
            from ..utils.display import Display

            if suppress_extended_error:
                self._show_content = False

            Display().deprecated(
                msg=f"The `suppress_extended_error` argument to `{type(self).__name__}` is deprecated.",
                version="2.23",
                help_text="Use `show_content=False` instead.",
            )

    @property
    def _original_message(self) -> str:
        return self._message

    @property
    def message(self) -> str:
        """
        Return the original message with cause message(s) appended.
        The cause will not be followed on any `AnsibleError` with `_include_cause_message=False`.
        """
        return _error_utils.format_exception_message(self)

    @message.setter
    def message(self, val) -> None:
        self._message = val

    @property
    def _formatted_source_context(self) -> str | None:
        with _error_utils.RedactAnnotatedSourceContext.when(not self._show_content):
            if source_context := _error_utils.SourceContext.from_value(self.obj):
                return str(source_context)

        return None

    @property
    def _help_text(self) -> str | None:
        return self._help_text_value

    @_help_text.setter
    def _help_text(self, value: str | None) -> None:
        self._help_text_value = value

    def __str__(self) -> str:
        return self.message

    def __getstate__(self) -> dict[str, t.Any]:
        """Augment object.__getstate__ to preserve additional values not represented in BaseException.__dict__."""
        state = t.cast(dict[str, t.Any], super().__getstate__())
        state.update(
            args=self.args,
            __cause__=self.__cause__,
            __context__=self.__context__,
            __suppress_context__=self.__suppress_context__,
        )

        return state

    def __reduce__(self) -> tuple[t.Callable, tuple[type], dict[str, t.Any]]:
        """
        Enable copy/pickle of AnsibleError derived types by correcting for BaseException's ancient C __reduce__ impl that:

        * requires use of a type constructor with positional args
        * assumes positional args are passed through from the derived type __init__ to BaseException.__init__ unmodified
        * does not propagate args/__cause__/__context__/__suppress_context__

        NOTE: This does not preserve the dunder attributes on non-AnsibleError derived cause/context exceptions.
              As a result, copy/pickle will discard chained exceptions after the first non-AnsibleError cause/context.
        """
        return type(self).__new__, (type(self),), self.__getstate__()


class AnsibleUndefinedConfigEntry(AnsibleError):
    """The requested config entry is not defined."""


class AnsibleTaskError(AnsibleError):
    """Task execution failed; provides contextual information about the task."""

    _default_message = 'Task failed.'


class AnsiblePromptInterrupt(AnsibleError):
    """User interrupt."""


class AnsiblePromptNoninteractive(AnsibleError):
    """Unable to get user input."""


class AnsibleAssertionError(AnsibleError, AssertionError):
    """Invalid assertion."""


class AnsibleOptionsError(AnsibleError):
    """Invalid options were passed."""

    # FIXME: This exception is used for many non-CLI related errors.
    #          The few cases which are CLI related should really be handled by argparse instead, at which point the exit code here can be removed.
    _exit_code = ExitCode.INVALID_CLI_OPTION


class AnsibleRequiredOptionError(AnsibleOptionsError):
    """Bad or incomplete options passed."""


class AnsibleParserError(AnsibleError):
    """A playbook or data file could not be parsed."""

    _exit_code = ExitCode.PARSER_ERROR


class AnsibleFieldAttributeError(AnsibleParserError):
    """Errors caused during field attribute processing."""


class AnsibleJSONParserError(AnsibleParserError):
    """JSON-specific parsing failure wrapping an exception raised by the JSON parser."""

    _default_message = 'JSON parsing failed.'
    _include_cause_message = False  # hide the underlying cause message, it's included by `handle_exception` as needed

    @classmethod
    def handle_exception(cls, exception: Exception, origin: _tags.Origin) -> t.NoReturn:
        if isinstance(exception, JSONDecodeError):
            origin = origin.replace(line_num=exception.lineno, col_num=exception.colno)

        message = str(exception)

        error = cls(message, obj=origin)

        raise error from exception


class AnsibleInternalError(AnsibleError):
    """Internal safeguards tripped, something happened in the code that should never happen."""


class AnsibleRuntimeError(AnsibleError):
    """Ansible had a problem while running a playbook."""


class AnsibleModuleError(AnsibleRuntimeError):
    """A module failed somehow."""


class AnsibleConnectionFailure(AnsibleRuntimeError, _error_utils.ContributesToTaskResult):
    """
    The transport / connection_plugin had a fatal error.

    This exception provides a result dictionary via the ContributesToTaskResult mixin.
    """

    @property
    def result_contribution(self) -> t.Mapping[str, object]:
        return dict(unreachable=True)

    @property
    def omit_failed_key(self) -> bool:
        return True


class AnsibleAuthenticationFailure(AnsibleConnectionFailure):
    """Invalid username/password/key."""

    _default_message = "Failed to authenticate."


class AnsibleCallbackError(AnsibleRuntimeError):
    """A callback failure."""


class AnsibleTemplateError(AnsibleRuntimeError):
    """A template related error."""


class TemplateTrustCheckFailedError(AnsibleTemplateError):
    """Raised when processing was requested on an untrusted template or expression."""

    _default_message = 'Encountered untrusted template or expression.'
    _default_help_text = ('Templates and expressions must be defined by trusted sources such as playbooks or roles, '
                          'not untrusted sources such as module results.')


class AnsibleTemplateTransformLimitError(AnsibleTemplateError):
    """The internal template transform limit was exceeded."""

    _default_message = "Template transform limit exceeded."


class AnsibleTemplateSyntaxError(AnsibleTemplateError):
    """A syntax error was encountered while parsing a Jinja template or expression."""


class AnsibleBrokenConditionalError(AnsibleTemplateError):
    """A broken conditional with non-boolean result was used."""

    _default_help_text = 'Broken conditionals can be temporarily allowed with the `ALLOW_BROKEN_CONDITIONALS` configuration option.'


class AnsibleUndefinedVariable(AnsibleTemplateError):
    """An undefined variable was encountered while processing a template or expression."""


class AnsibleValueOmittedError(AnsibleTemplateError):
    """
    Raised when the result of a template operation was the Omit singleton. This exception purposely does
    not derive from AnsibleError to avoid elision of the traceback, since uncaught errors of this type always
    indicate a bug.
    """

    _default_message = "A template was resolved to an Omit scalar."
    _default_help_text = "Callers must be prepared to handle this value. This is most likely a bug in the code requesting templating."


class AnsibleTemplatePluginError(AnsibleTemplateError):
    """An error sourced by a template plugin (lookup/filter/test)."""


# deprecated: description='add deprecation warnings for these aliases' core_version='2.23'
AnsibleFilterError = AnsibleTemplatePluginError
AnsibleLookupError = AnsibleTemplatePluginError


class AnsibleFileNotFound(AnsibleRuntimeError):
    """A file missing failure."""

    def __init__(self, message="", obj=None, show_content=True, suppress_extended_error=..., orig_exc=None, paths=None, file_name=None):

        self.file_name = file_name
        self.paths = paths

        if message:
            message += "\n"
        if self.file_name:
            message += "Could not find or access '%s'" % to_text(self.file_name)
        else:
            message += "Could not find file"

        if self.paths and isinstance(self.paths, _c.Sequence):
            searched = to_text('\n\t'.join(self.paths))
            if message:
                message += "\n"
            message += "Searched in:\n\t%s" % searched

        message += " on the Ansible Controller.\nIf you are using a module and expect the file to exist on the remote, see the remote_src option"

        super(AnsibleFileNotFound, self).__init__(message=message, obj=obj, show_content=show_content,
                                                  suppress_extended_error=suppress_extended_error, orig_exc=orig_exc)


class AnsibleAction(AnsibleRuntimeError, _error_utils.ContributesToTaskResult):
    """Base Exception for Action plugin flow control."""

    def __init__(self, message="", obj=None, show_content=True, suppress_extended_error=..., orig_exc=None, result=None):
        super().__init__(message=message, obj=obj, show_content=show_content, suppress_extended_error=suppress_extended_error, orig_exc=orig_exc)

        self._result = result or {}

    @property
    def result_contribution(self) -> _c.Mapping[str, object]:
        return self._result

    @property
    def result(self) -> dict[str, object]:
        """Backward compatibility property returning a mutable dictionary."""
        return dict(self.result_contribution)


class AnsibleActionSkip(AnsibleAction):
    """
    An action runtime skip.

    This exception provides a result dictionary via the ContributesToTaskResult mixin.
    """

    @property
    def result_contribution(self) -> _c.Mapping[str, object]:
        return self._result | dict(
            skipped=True,
            msg=self.message,
        )

    @property
    def omit_failed_key(self) -> bool:
        return True

    @property
    def omit_exception_key(self) -> bool:
        return True


class AnsibleActionFail(AnsibleAction):
    """
    An action runtime failure.

    This exception provides a result dictionary via the ContributesToTaskResult mixin.
    """

    @property
    def result_contribution(self) -> _c.Mapping[str, object]:
        return self._result | dict(
            failed=True,
            msg=self.message,
        )


class _ActionDone(AnsibleAction):
    """
    Imports as `_AnsibleActionDone` are deprecated. An action runtime early exit.

    This exception provides a result dictionary via the ContributesToTaskResult mixin.
    """

    @property
    def omit_failed_key(self) -> bool:
        return not self._result.get('failed')

    @property
    def omit_exception_key(self) -> bool:
        return not self._result.get('failed')


class AnsiblePluginError(AnsibleError):
    """Base class for Ansible plugin-related errors that do not need AnsibleError contextual data."""

    def __init__(self, message: str | None = None, plugin_load_context: _t_loader.PluginLoadContext | None = None, help_text: str | None = None) -> None:
        super(AnsiblePluginError, self).__init__(message, help_text=help_text)

        self.plugin_load_context = plugin_load_context


class AnsiblePluginRemovedError(AnsiblePluginError):
    """A requested plugin has been removed."""


class AnsiblePluginCircularRedirect(AnsiblePluginError):
    """A cycle was detected in plugin redirection."""


class AnsibleCollectionUnsupportedVersionError(AnsiblePluginError):
    """A collection is not supported by this version of Ansible."""


class AnsibleTypeError(AnsibleRuntimeError, TypeError):
    """Ansible-augmented TypeError subclass."""


class AnsiblePluginNotFound(AnsiblePluginError):
    """Indicates we did not find an Ansible plugin."""


class AnsibleConditionalError(AnsibleRuntimeError):
    """Errors related to failed conditional expression evaluation."""


class AnsibleVariableTypeError(AnsibleRuntimeError):
    """An error due to attempted storage of an unsupported variable type."""

    @classmethod
    def from_value(cls, *, obj: t.Any) -> t.Self:
        # avoid an incorrect error message when `obj` is a type
        type_name = type(obj).__name__ if isinstance(obj, type) else native_type_name(obj)

        return cls(message=f'Type {type_name!r} is unsupported for variable storage.', obj=obj)


def __getattr__(name: str) -> t.Any:
    """Inject import-time deprecation warnings."""
    from ..utils.display import Display

    match name:
        case 'AnsibleFilterTypeError':
            Display().deprecated(
                msg=f"Importing {name!r} is deprecated.",
                help_text=f"Import {AnsibleTypeError.__name__!r} instead.",
                version="2.23",
            )

            return AnsibleTypeError

        case '_AnsibleActionDone':
            Display().deprecated(
                msg=f"Importing {name!r} is deprecated.",
                help_text="Return directly from action plugins instead.",
                version="2.23",
            )

            return _ActionDone

    raise AttributeError(f'module {__name__!r} has no attribute {name!r}')



# -*- coding: utf-8 -*-

# Copyright: (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import annotations


class ModuleDocFragment(object):

    # inventory cache
    DOCUMENTATION = r"""
options:
  cache:
    description:
      - Toggle to enable/disable the caching of the inventory's source data, requires a cache plugin setup to work.
    type: bool
    default: no
    env:
      - name: ANSIBLE_INVENTORY_CACHE
    ini:
      - section: inventory
        key: cache
  cache_plugin:
    description:
      - Cache plugin to use for the inventory's source data.
    type: str
    default: memory
    env:
      - name: ANSIBLE_CACHE_PLUGIN
      - name: ANSIBLE_INVENTORY_CACHE_PLUGIN
    ini:
      - section: defaults
        key: fact_caching
      - section: inventory
        key: cache_plugin
  cache_timeout:
    description:
      - Cache duration in seconds.
    default: 3600
    type: int
    env:
      - name: ANSIBLE_CACHE_PLUGIN_TIMEOUT
      - name: ANSIBLE_INVENTORY_CACHE_TIMEOUT
    ini:
      - section: defaults
        key: fact_caching_timeout
      - section: inventory
        key: cache_timeout
  cache_connection:
    description:
      - Cache connection data or path, read cache plugin documentation for specifics.
    type: str
    env:
      - name: ANSIBLE_CACHE_PLUGIN_CONNECTION
      - name: ANSIBLE_INVENTORY_CACHE_CONNECTION
    ini:
      - section: defaults
        key: fact_caching_connection
      - section: inventory
        key: cache_connection
  cache_prefix:
    description:
      - Prefix to use for cache plugin files/tables.
    default: ansible_inventory_
    env:
      - name: ANSIBLE_CACHE_PLUGIN_PREFIX
      - name: ANSIBLE_INVENTORY_CACHE_PLUGIN_PREFIX
    ini:
      - section: defaults
        key: fact_caching_prefix
      - section: inventory
        key: cache_prefix
"""



# (c) 2017 Ansible By Red Hat
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import annotations

import collections.abc as c

from ansible.playbook import Play
from ansible.playbook.block import Block
from ansible.playbook.role import Role
from ansible.playbook.task import Task
from ansible._internal._templating._engine import TemplateEngine
from ansible.utils.display import Display

display = Display()


def get_reserved_names(include_private: bool = True) -> set[str]:
    """ this function returns the list of reserved names associated with play objects"""

    public = set(TemplateEngine().environment.globals.keys())
    private = set()

    # FIXME: find a way to 'not hardcode', possibly need role deps/includes
    class_list = [Play, Role, Block, Task]

    for aclass in class_list:
        # build ordered list to loop over and dict with attributes
        for name, attr in aclass.fattributes.items():
            if attr.private:
                private.add(name)
            else:
                public.add(name)

    # local_action is implicit with action
    if 'action' in public:
        public.add('local_action')

    # loop implies with_
    # FIXME: remove after with_ is not only deprecated but removed
    if 'loop' in private or 'loop' in public:
        public.add('with_')

    if include_private:
        result = public.union(private)
    else:
        result = public

    result.discard('gather_subset')

    return result


def warn_if_reserved(myvars: c.Iterable[str], additional: c.Iterable[str] | None = None) -> None:
    """Issue a warning for any variable which conflicts with an internally reserved name."""
    if additional is None:
        reserved = _RESERVED_NAMES
    else:
        reserved = _RESERVED_NAMES.union(additional)

    varnames = set(myvars)
    varnames.discard('vars')  # we add this one internally, so safe to ignore

    if conflicts := varnames.intersection(reserved):
        # Ensure the varname used for obj is the tagged one from myvars and not the untagged one from reserved.
        # This can occur because tags do not affect value equality, and intersection can return values from either the left or right side.
        for varname in (name for name in myvars if name in conflicts):
            display.warning(f'Found variable using reserved name {varname!r}.', obj=varname)


def is_reserved_name(name: str) -> bool:
    return name in _RESERVED_NAMES


_RESERVED_NAMES = frozenset(get_reserved_names())



# -*- coding: utf-8 -*-
# (c) 2017, Adrian Likins <alikins@redhat.com>
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import annotations

import os
import pytest

import unittest
from unittest.mock import patch, MagicMock
from units.mock.vault_helper import TextVaultSecret

from ansible import context, errors
from ansible.cli.vault import VaultCLI
from ansible.module_utils.common.text.converters import to_text
from ansible.utils import context_objects as co


# TODO: make these tests assert something, likely by verifying
#       mock calls


@pytest.fixture(autouse=True)
def reset_cli_args():
    co.GlobalCLIArgs._Singleton__instance = None
    yield
    co.GlobalCLIArgs._Singleton__instance = None


class TestVaultCli(unittest.TestCase):
    def setUp(self):
        self.tty_patcher = patch('ansible.cli.sys.stdin.isatty', return_value=False)
        self.mock_isatty = self.tty_patcher.start()

    def tearDown(self):
        self.tty_patcher.stop()

    def test_parse_empty(self):
        cli = VaultCLI(['vaultcli'])
        self.assertRaises(SystemExit,
                          cli.parse)

    # FIXME: something weird seems to be afoot when parsing actions
    # cli = VaultCLI(args=['view', '/dev/null/foo', 'mysecret3'])
    # will skip '/dev/null/foo'. something in cli.CLI.set_action() ?
    #   maybe we self.args gets modified in a loop?
    def test_parse_view_file(self):
        cli = VaultCLI(args=['ansible-vault', 'view', '/dev/null/foo'])
        cli.parse()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    def test_view_missing_file_no_secret(self, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = []
        cli = VaultCLI(args=['ansible-vault', 'view', '/dev/null/foo'])
        cli.parse()
        self.assertRaisesRegex(errors.AnsibleOptionsError,
                               "A vault password is required to use Ansible's Vault",
                               cli.run)

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    def test_encrypt_missing_file_no_secret(self, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = []
        cli = VaultCLI(args=['ansible-vault', 'encrypt', '/dev/null/foo'])
        cli.parse()
        self.assertRaisesRegex(errors.AnsibleOptionsError,
                               "A vault password is required to use Ansible's Vault",
                               cli.run)

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_encrypt(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'encrypt', '/dev/null/foo'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_encrypt_string(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'encrypt_string',
                             'some string to encrypt'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    @patch('ansible.cli.vault.display.prompt', return_value='a_prompt')
    def test_encrypt_string_prompt(self, mock_display, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault',
                             'encrypt_string',
                             '--prompt',
                             '--show-input',
                             'some string to encrypt'])
        cli.parse()
        cli.run()
        args, kwargs = mock_display.call_args
        assert kwargs["private"] is False

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    @patch('ansible.cli.vault.display.prompt', return_value='a_prompt')
    def test_shadowed_encrypt_string_prompt(self, mock_display, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault',
                             'encrypt_string',
                             '--prompt'])
        cli.parse()
        cli.run()
        args, kwargs = mock_display.call_args
        assert kwargs["private"]

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    @patch('ansible.cli.vault.display.prompt', return_value='a_prompt')
    def test_shadowed_encrypt_string_prompt_plus(self, mock_display, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault',
                             'encrypt_string',
                             'some string to encrypt',
                             '--prompt'])
        cli.parse()
        cli.run()
        args, kwargs = mock_display.call_args
        assert kwargs["private"]

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    @patch('ansible.cli.vault.sys.stdin.read', return_value='This is data from stdin')
    def test_encrypt_string_stdin(self, mock_stdin_read, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault',
                             'encrypt_string',
                             '--stdin-name',
                             'the_var_from_stdin',
                             '-'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_encrypt_string_names(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'encrypt_string',
                             '--name', 'foo1',
                             '--name', 'foo2',
                             'some string to encrypt'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_encrypt_string_more_args_than_names(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'encrypt_string',
                             '--name', 'foo1',
                             'some string to encrypt',
                             'other strings',
                             'a few more string args'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_create(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'create', '/dev/null/foo'])
        cli.parse()
        self.assertRaisesRegex(errors.AnsibleOptionsError,
                               "not a tty, editor cannot be opened",
                               cli.run)

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_create_skip_tty_check(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'create', '--skip-tty-check', '/dev/null/foo'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_create_with_tty(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        self.tty_stdout_patcher = patch('ansible.cli.sys.stdout.isatty', return_value=True)
        self.tty_stdout_patcher.start()
        cli = VaultCLI(args=['ansible-vault', 'create', '/dev/null/foo'])
        cli.parse()
        cli.run()
        self.tty_stdout_patcher.stop()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_edit(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'edit', '/dev/null/foo'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_decrypt(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'decrypt', '/dev/null/foo'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_view(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'view', '/dev/null/foo'])
        cli.parse()
        cli.run()

    @patch('ansible.cli.vault.VaultCLI.setup_vault_secrets')
    @patch('ansible.cli.vault.VaultEditor')
    def test_rekey(self, mock_vault_editor, mock_setup_vault_secrets):
        mock_setup_vault_secrets.return_value = [('default', TextVaultSecret('password'))]
        cli = VaultCLI(args=['ansible-vault', 'rekey', '/dev/null/foo'])
        cli.parse()
        cli.run()


@pytest.mark.parametrize('cli_args, expected', [
    (['ansible-vault', 'view', 'vault.txt'], 0),
    (['ansible-vault', 'view', 'vault.txt', '-vvv'], 3),
    (['ansible-vault', 'view', 'vault.txt', '-vv'], 2),
])
def test_verbosity_arguments(cli_args, expected, tmp_path_factory, monkeypatch):
    # Add a password file so we don't get a prompt in the test
    test_dir = to_text(tmp_path_factory.mktemp('test-ansible-vault'))
    pass_file = os.path.join(test_dir, 'pass.txt')
    with open(pass_file, 'w') as pass_fd:
        pass_fd.write('password')

    cli_args.extend(['--vault-id', pass_file])

    # Mock out the functions so we don't actually execute anything
    for func_name in [f for f in dir(VaultCLI) if f.startswith("execute_")]:
        monkeypatch.setattr(VaultCLI, func_name, MagicMock())

    cli = VaultCLI(args=cli_args)
    cli.run()

    assert context.CLIARGS['verbosity'] == expected



from __future__ import annotations

from ansible.plugins.action import ActionBase
from ansible._internal._ssh._ssh_agent import PublicKeyMsg
from ansible.module_utils.common.text.converters import to_bytes, to_text


from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric.rsa import generate_private_key as rsa_generate_private_key
from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PrivateKey
from cryptography.hazmat.primitives.asymmetric.dsa import generate_private_key as dsa_generate_private_key
from cryptography.hazmat.primitives.asymmetric.ec import SECP384R1, generate_private_key as ecdsa_generate_private_key


class ActionModule(ActionBase):

    def run(self, tmp=None, task_vars=None):
        results = super(ActionModule, self).run(tmp, task_vars)
        del tmp  # tmp no longer has any effect
        match self._task.args.get('type'):
            case 'ed25519':
                private_key = Ed25519PrivateKey.generate()
            case 'rsa':
                private_key = rsa_generate_private_key(65537, 4096)
            case 'dsa':
                private_key = dsa_generate_private_key(1024)
            case 'ecdsa':
                private_key = ecdsa_generate_private_key(SECP384R1())
            case _:
                return {'failed': True, 'msg': 'not implemented'}

        public_key = private_key.public_key()
        public_key_msg = PublicKeyMsg.from_public_key(public_key)

        if not (passphrase := self._task.args.get('passphrase')):
            encryption_algorithm = serialization.NoEncryption()
        else:
            encryption_algorithm = serialization.BestAvailableEncryption(
                to_bytes(passphrase)
            )

        return {
            'changed': True,
            'private_key': to_text(private_key.private_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PrivateFormat.OpenSSH,
                encryption_algorithm=encryption_algorithm,
            )),
            'public_key': to_text(public_key.public_bytes(
                encoding=serialization.Encoding.OpenSSH,
                format=serialization.PublicFormat.OpenSSH,
            )),
            'fingerprint': f'SHA256:{public_key_msg.fingerprint}',
        }



# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

import typing as t

import pytest
import pytest_mock

from ansible.utils.display import Display
from ansible.utils.listify import listify_lookup_plugin_terms


@pytest.mark.parametrize("test_input, expected", (
    ([], []),
    ("foo", ["foo"]),
    (["foo"], ["foo"]),
), ids=str)
def test_listify_lookup_plugin_terms(test_input: t.Any, expected: t.Any, mocker: pytest_mock.MockerFixture) -> None:
    with mocker.patch.object(Display(), 'deprecated'):  # as deprecated:
        assert listify_lookup_plugin_terms(test_input) == expected

    # deprecated: description="Calling listify_lookup_plugin_terms function is not necessary; the function should be deprecated." core_version="2.23"
    # deprecated.assert_called_once_with(
    #     msg='"listify_lookup_plugin_terms" is obsolete and in most cases unnecessary',
    #     version='2.23',
    # )



# -*- coding: utf-8 -*-
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations


DOCUMENTATION = r"""
---
module: willremove
version_added: histerical
short_description: does nothing
description: does nothing, this is deprecation test
deprecated:
    removed_in: '3.30'
    why: cause i wanna!
    alternatives: no soup for you!
options:
  one:
    description:
    - first option
    type: bool
    default: no
  two:
    description:
    - second option
notes:
- Just noop to test module deprecation
seealso:
- module: removeoption
author:
- Ansible Core Team
attributes:
    action:
      support: full
    async:
      support: full
    bypass_host_loop:
      support: none
    check_mode:
      support: full
    diff_mode:
      support: none
    platform:
      platforms: all
"""

EXAMPLES = r"""
- name: useless
  willremove:
    one: true
    two: /etc/file.conf
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule


def main():
    module = AnsibleModule(
        argument_spec=dict(
            one=dict(type='bool', default='no'),
            two=dict(type='str'),
        ),
        supports_check_mode=True
    )

    one = module.params['one']
    two = module.params['two']

    result = {'yolo': 'lola'}

    module.exit_json(**result)


if __name__ == '__main__':
    main()



from __future__ import annotations

import sys

prompts = sys.argv[1:] or ['foo']

# latin1 encoded bytes
# to ensure pexpect doesn't have any encoding errors
data = b'premi\xe8re is first\npremie?re is slightly different\n????????? is Cyrillic\n? am Deseret\n'

sys.stdout.buffer.write(data)
print()

for prompt in prompts:
    user_input = input(prompt)
    print(user_input)



# Copyright (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

DOCUMENTATION = """
    inventory: yaml
    version_added: "2.4"
    short_description: Uses a specific YAML file as an inventory source.
    description:
        - "YAML-based inventory, should start with the C(all) group and contain hosts/vars/children entries."
        - Host entries can have sub-entries defined, which will be treated as variables.
        - Vars entries are normal group vars.
        - "Children are 'child groups', which can also have their own vars/hosts/children and so on."
        - File MUST have a valid extension, defined in configuration.
    notes:
        - If you want to set vars for the C(all) group inside the inventory file, the C(all) group must be the first entry in the file.
        - Enabled in configuration by default.
    options:
      yaml_extensions:
        description: list of 'valid' extensions for files containing YAML
        type: list
        default: ['.yaml', '.yml', '.json']
        env:
          - name: ANSIBLE_YAML_FILENAME_EXT
          - name: ANSIBLE_INVENTORY_PLUGIN_EXTS
        ini:
          - key: yaml_valid_extensions
            section: defaults
          - section: inventory_plugin_yaml
            key: yaml_valid_extensions

"""
EXAMPLES = """
all: # keys must be unique, i.e. only one 'hosts' per group
    hosts:
        test1:
        test2:
            host_var: value
    vars:
        group_all_var: value
    children:   # key order does not matter, indentation does
        other_group:
            children:
                group_x:
                    hosts:
                        test5
            vars:
                g2_var2: value3
            hosts:
                test4:
                    ansible_host: 127.0.0.1
        last_group:
            hosts:
                test1 # same host as above, additional group membership
            vars:
                group_last_var: value
"""

import os

from collections.abc import MutableMapping

from ansible.errors import AnsibleError, AnsibleParserError
from ansible.module_utils.six import string_types
from ansible.module_utils.common.text.converters import to_native, to_text
from ansible.plugins.inventory import BaseFileInventoryPlugin

NoneType = type(None)


class InventoryModule(BaseFileInventoryPlugin):

    NAME = 'yaml'

    def __init__(self):

        super(InventoryModule, self).__init__()

    def verify_file(self, path):

        valid = False
        if super(InventoryModule, self).verify_file(path):
            file_name, ext = os.path.splitext(path)
            if not ext or ext in self.get_option('yaml_extensions'):
                valid = True
        return valid

    def parse(self, inventory, loader, path, cache=True):
        """ parses the inventory file """

        super(InventoryModule, self).parse(inventory, loader, path)
        self.set_options()

        try:
            data = self.loader.load_from_file(path, cache='none')
        except Exception as e:
            raise AnsibleParserError(e)

        if not data:
            raise AnsibleParserError('Parsed empty YAML file')
        elif not isinstance(data, MutableMapping):
            raise AnsibleParserError('YAML inventory has invalid structure, it should be a dictionary, got: %s' % type(data))
        elif data.get('plugin'):
            raise AnsibleParserError('Plugin configuration YAML file, not YAML inventory')

        # We expect top level keys to correspond to groups, iterate over them
        # to get host, vars and subgroups (which we iterate over recursively)
        if isinstance(data, MutableMapping):
            for group_name in data:
                self._parse_group(group_name, data[group_name])
        else:
            raise AnsibleParserError("Invalid data from file, expected dictionary and got:\n\n%s" % to_native(data))

    def _parse_group(self, group, group_data):

        if isinstance(group_data, (MutableMapping, NoneType)):

            try:
                self.inventory.add_group(group)
            except AnsibleError as e:
                raise AnsibleParserError("Unable to add group %s: %s" % (group, to_text(e)))

            if group_data is not None:
                # make sure they are dicts
                for section in ['vars', 'children', 'hosts']:
                    if section in group_data:
                        # convert strings to dicts as these are allowed
                        if isinstance(group_data[section], string_types):
                            group_data[section] = {group_data[section]: None}

                        if not isinstance(group_data[section], (MutableMapping, NoneType)):
                            raise AnsibleParserError('Invalid "%s" entry for "%s" group, requires a dictionary, found "%s" instead.' %
                                                     (section, group, type(group_data[section])))

                for key in group_data:

                    if not isinstance(group_data[key], (MutableMapping, NoneType)):
                        self.display.warning('Skipping key (%s) in group (%s) as it is not a mapping, it is a %s' % (key, group, type(group_data[key])))
                        continue

                    if isinstance(group_data[key], NoneType):
                        self.display.vvv('Skipping empty key (%s) in group (%s)' % (key, group))
                    elif key == 'vars':
                        for var in group_data[key]:
                            self.inventory.set_variable(group, var, group_data[key][var])
                    elif key == 'children':
                        for subgroup in group_data[key]:
                            self._parse_group(subgroup, group_data[key][subgroup])
                            self.inventory.add_child(group, subgroup)

                    elif key == 'hosts':
                        for host_pattern in group_data[key]:
                            hosts, port = self._parse_host(host_pattern)
                            self._populate_host_vars(hosts, group_data[key][host_pattern] or {}, group, port)
                    else:
                        self.display.warning('Skipping unexpected key (%s) in group (%s), only "vars", "children" and "hosts" are valid' % (key, group))

        else:
            self.display.warning("Skipping '%s' as this is not a valid group definition" % group)

    def _parse_host(self, host_pattern):
        """
        Each host key can be a pattern, try to process it and add variables as needed
        """
        (hostnames, port) = self._expand_hostpattern(host_pattern)

        return hostnames, port



from __future__ import annotations

from ansible.plugins.action import ActionBase


class ActionModule(ActionBase):
    def run(self, tmp=None, task_vars=None):
        result = super().run(tmp, task_vars)

        if self._task.args and 'msg' in self._task.args:
            msg = self._task.args.get('msg')
        else:
            msg = "Hello overridden world!"

        result['changes'] = False
        result['msg'] = msg
        return result



from __future__ import annotations

import re

import typing as t

from yaml import MarkedYAMLError
from yaml.constructor import ConstructorError

from ansible._internal._errors import _error_utils
from ansible.errors import AnsibleParserError
from ansible._internal._datatag._tags import Origin


class AnsibleConstructorError(ConstructorError):
    """Ansible-specific ConstructorError used to bypass exception analysis during wrapping in AnsibleYAMLParserError."""


class AnsibleYAMLParserError(AnsibleParserError):
    """YAML-specific parsing failure wrapping an exception raised by the YAML parser."""

    _default_message = 'YAML parsing failed.'

    _include_cause_message = False  # hide the underlying cause message, it's included by `handle_exception` as needed

    _formatted_source_context_value: str | None = None

    @property
    def _formatted_source_context(self) -> str | None:
        return self._formatted_source_context_value

    @classmethod
    def handle_exception(cls, exception: Exception, origin: Origin) -> t.NoReturn:
        if isinstance(exception, MarkedYAMLError):
            origin = origin.replace(line_num=exception.problem_mark.line + 1, col_num=exception.problem_mark.column + 1)

        source_context = _error_utils.SourceContext.from_origin(origin)

        target_line = source_context.target_line or ''  # for these cases, we don't need to distinguish between None and empty string

        message: str | None = None
        help_text = None

        # FIXME: Do all this by walking the parsed YAML doc stream. Using regexes is a dead-end; YAML's just too flexible to not have a
        #  raft of false-positives and corner cases. If we directly consume either the YAML parse stream or override the YAML composer, we can
        #  better catch these things without worrying about duplicating YAML's scalar parsing logic around quoting/escaping. At first, we can
        #  replace the regex logic below with tiny special-purpose parse consumers to catch specific issues, but ideally, we could do a lot of this
        #  inline with the actual doc parse, since our rules are a lot more strict than YAML's (eg, no support for non-scalar keys), and a lot of the
        #  problem cases where that comes into play are around expression quoting and Jinja {{ syntax looking like weird YAML values we don't support.
        #  Some common examples, where -> is "what YAML actually sees":
        #  foo: {{ bar }} -> {"foo": {{"bar": None}: None}} - a mapping with a mapping as its key (legal YAML, but not legal Python/Ansible)
        #
        #  - copy: src=foo.txt  # kv syntax (kv could be on following line(s), too- implicit multi-line block scalar)
        #      dest: bar.txt  # orphaned mapping, since the value of `copy` is the scalar "src=foo.txt"
        #
        #  - msg == "Error: 'dude' was not found"  # unquoted scalar has a : in it -> {'msg == "Error"': 'dude'} [ was not found" ] is garbage orphan scalar

        # noinspection PyUnboundLocalVariable
        if not isinstance(exception, MarkedYAMLError):
            pass  # unexpected exception, don't use special analysis of exception

        elif isinstance(exception, AnsibleConstructorError):
            pass  # raised internally by ansible code, don't use special analysis of exception

        # Check for tabs.
        # There may be cases where there is a valid tab in a line that has other errors.
        # That's OK, users should "fix" their tab usage anyway -- at which point later error handling logic will hopefully find the real issue.
        elif (tab_idx := target_line.find('\t')) >= 0:
            source_context = _error_utils.SourceContext.from_origin(origin.replace(col_num=tab_idx + 1))
            message = "Tabs are usually invalid in YAML."

        # Check for unquoted templates.
        elif match := re.search(r'^\s*(?:-\s+)*(?:[\w\s]+:\s+)?(?P<value>\{\{.*}})', target_line):
            source_context = _error_utils.SourceContext.from_origin(origin.replace(col_num=match.start('value') + 1))
            message = 'This may be an issue with missing quotes around a template block.'
            # FIXME: Use the captured value to show the actual fix required.
            help_text = """
For example:

    raw: {{ some_var }}

Should be:

    raw: "{{ some_var }}"
"""

        # Check for common unquoted colon mistakes.
        elif (
            # ignore lines starting with only whitespace and a colon
            not target_line.lstrip().startswith(':')
            # find the value after list/dict preamble
            and (value_match := re.search(r'^\s*(?:-\s+)*(?:[\w\s\[\]{}]+:\s+)?(?P<value>.*)$', target_line))
            # ignore properly quoted values
            and (target_fragment := _replace_quoted_value(value_match.group('value')))
            # look for an unquoted colon in the value
            and (colon_match := re.search(r':($| )', target_fragment))
        ):
            source_context = _error_utils.SourceContext.from_origin(origin.replace(col_num=value_match.start('value') + colon_match.start() + 1))
            message = 'Colons in unquoted values must be followed by a non-space character.'
            # FIXME: Use the captured value to show the actual fix required.
            help_text = """
For example:

    raw: echo 'name: ansible'

Should be:

    raw: "echo 'name: ansible'"
"""

        # Check for common quoting mistakes.
        elif match := re.search(r'^\s*(?:-\s+)*(?:[\w\s]+:\s+)?(?P<value>[\"\'].*?\s*)$', target_line):
            suspected_value = match.group('value')
            first, last = suspected_value[0], suspected_value[-1]

            if first != last:  # "foo" in bar
                source_context = _error_utils.SourceContext.from_origin(origin.replace(col_num=match.start('value') + 1))
                message = 'Values starting with a quote must end with the same quote.'
                # FIXME: Use the captured value to show the actual fix required, and use that same logic to improve the origin further.
                help_text = """
For example:

    raw: "foo" in bar

Should be:

    raw: '"foo" in bar'
"""
            elif first == last and target_line.count(first) > 2:  # "foo" and "bar"
                source_context = _error_utils.SourceContext.from_origin(origin.replace(col_num=match.start('value') + 1))
                message = 'Values starting with a quote must end with the same quote, and not contain that quote.'
                # FIXME: Use the captured value to show the actual fix required, and use that same logic to improve the origin further.
                help_text = """
For example:

    raw: "foo" in "bar"

Should be:

    raw: '"foo" in "bar"'
"""

        if not message:
            if isinstance(exception, MarkedYAMLError):
                # marked YAML error, pull out the useful messages while omitting the noise
                message = ' '.join(filter(None, (exception.context, exception.problem, exception.note)))
                message = message.strip()
                message = f'{message[0].upper()}{message[1:]}'

                if not message.endswith('.'):
                    message += '.'
            else:
                # unexpected error, use the exception message (normally hidden by overriding include_cause_message)
                message = str(exception)

            message = re.sub(r'\s+', ' ', message).strip()

        error = cls(message, obj=source_context.origin)
        error._formatted_source_context_value = str(source_context)
        error._help_text = help_text

        raise error from exception


def _replace_quoted_value(value: str, replacement='.') -> str:
    return re.sub(r"""^\s*('[^']*'|"[^"]*")\s*$""", lambda match: replacement * len(match.group(0)), value)



# (c) 2012-2014, Michael DeHaan <michael.dehaan@gmail.com>
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import annotations

import pytest

from ansible.errors import AnsibleAssertionError, AnsibleParserError
from ansible.playbook.block import Block
from ansible.playbook.play import Play
from ansible.playbook.role import Role
from ansible.playbook.task import Task

from units.mock.loader import DictDataLoader


def test_play_with_hosts_string():
    p = Play.load({'hosts': 'foo'})

    assert p.get_name() == 'foo'


def test_basic_play():
    p = Play.load(dict(
        name="test play",
        hosts=['foo'],
        gather_facts=False,
        connection='local',
        remote_user="root",
        become=True,
        become_user="testing",
    ))

    assert p.name == 'test play'
    assert p.hosts == ['foo']
    assert p.connection == 'local'


def test_play_with_remote_user():
    p = Play.load(dict(
        name="test play",
        hosts=['foo'],
        user="testing",
        gather_facts=False,
    ))

    assert p.remote_user == "testing"


def test_play_with_user_conflict():
    play_data = dict(
        name="test play",
        hosts=['foo'],
        user="testing",
        remote_user="testing",
    )

    with pytest.raises(AnsibleParserError):
        Play.load(play_data)


def test_play_with_bad_ds_type():
    play_data = []
    with pytest.raises(AnsibleAssertionError, match=r"while preprocessing data \(\[\]\), ds should be a dict but was a <(?:class|type) 'list'>"):
        Play.load(play_data)


def test_play_with_tasks():
    p = Play.load(dict(
        name="test play",
        hosts=['foo'],
        gather_facts=False,
        tasks=[dict(action='shell echo "hello world"')],
    ))

    assert len(p.tasks) == 1
    assert isinstance(p.tasks[0], Block)
    assert p.tasks[0].has_tasks() is True


def test_play_with_handlers():
    p = Play.load(dict(
        name="test play",
        hosts=['foo'],
        gather_facts=False,
        handlers=[dict(action='shell echo "hello world"')],
    ))

    assert len(p.handlers) >= 1
    assert len(p.get_handlers()) >= 1
    assert isinstance(p.handlers[0], Block)
    assert p.handlers[0].has_tasks() is True


def test_play_with_pre_tasks():
    p = Play.load(dict(
        name="test play",
        hosts=['foo'],
        gather_facts=False,
        pre_tasks=[dict(action='shell echo "hello world"')],
    ))

    assert len(p.pre_tasks) >= 1
    assert isinstance(p.pre_tasks[0], Block)
    assert p.pre_tasks[0].has_tasks() is True

    assert len(p.get_tasks()) >= 1
    assert isinstance(p.get_tasks()[0][0], Task)
    assert p.get_tasks()[0][0].action == 'shell'


def test_play_with_post_tasks():
    p = Play.load(dict(
        name="test play",
        hosts=['foo'],
        gather_facts=False,
        post_tasks=[dict(action='shell echo "hello world"')],
    ))

    assert len(p.post_tasks) >= 1
    assert isinstance(p.post_tasks[0], Block)
    assert p.post_tasks[0].has_tasks() is True


def test_play_with_roles(mocker):
    mocker.patch('ansible.playbook.role.definition.RoleDefinition._load_role_path', return_value=('foo', '/etc/ansible/roles/foo'))
    fake_loader = DictDataLoader({
        '/etc/ansible/roles/foo/tasks.yml': """
        - name: role task
          shell: echo "hello world"
        """,
    })

    mock_var_manager = mocker.MagicMock()
    mock_var_manager.get_vars.return_value = {}

    p = Play.load(dict(
        name="test play",
        hosts=['foo'],
        gather_facts=False,
        roles=['foo'],
    ), loader=fake_loader, variable_manager=mock_var_manager)

    blocks = p.compile()
    assert len(blocks) > 1
    assert all(isinstance(block, Block) for block in blocks)
    assert isinstance(p.get_roles()[0], Role)


def test_play_compile():
    p = Play.load(dict(
        name="test play",
        hosts=['foo'],
        gather_facts=False,
        tasks=[dict(action='shell echo "hello world"')],
    ))

    blocks = p.compile()

    # with a single block, there will still be three
    # implicit meta flush_handler blocks inserted
    assert len(blocks) == 4


@pytest.mark.parametrize(
    'value, expected',
    (
        ('my_vars.yml', ['my_vars.yml']),
        (['my_vars.yml'], ['my_vars.yml']),
        (['my_vars1.yml', 'my_vars2.yml'], ['my_vars1.yml', 'my_vars2.yml']),
        (None, []),
    )
)
def test_play_with_vars_files(value, expected):
    play = Play.load({
        'name': 'Play with vars_files',
        'hosts': ['testhost1'],
        'vars_files': value,
    })

    assert play.vars_files == value
    assert play.get_vars_files() == expected


@pytest.mark.parametrize('value', ([], tuple(), set(), {}, '', None, False, 0))
def test_play_empty_hosts(value):
    with pytest.raises(AnsibleParserError, match='Hosts list cannot be empty'):
        Play.load({'hosts': value})


@pytest.mark.parametrize('value', ([None], (None,), ['one', None]))
def test_play_none_hosts(value):
    with pytest.raises(AnsibleParserError, match="Hosts list cannot contain values of 'None'"):
        Play.load({'hosts': value})


@pytest.mark.parametrize(
    'value',
    (
        {'one': None},
        {'one': 'two'},
        True,
        1,
        1.75,
    )
)
def test_play_invalid_hosts_sequence(value):
    with pytest.raises(AnsibleParserError, match='Hosts list must be a sequence or string'):
        Play.load({'hosts': value})


@pytest.mark.parametrize(
    'value',
    (
        [[1, 'two']],
        [{'one': None}],
        [set((None, 'one'))],
        ['one', 'two', {'three': None}],
        ['one', 'two', {'three': 'four'}],
    )
)
def test_play_invalid_hosts_value(value):
    with pytest.raises(AnsibleParserError, match='Hosts list contains an invalid host value'):
        Play.load({'hosts': value})


def test_play_with_vars():
    play = Play.load({}, vars={'var1': 'val1'})

    assert play.get_name() == ''
    assert play.vars == {'var1': 'val1'}
    assert play.get_vars() == {'var1': 'val1'}


def test_play_no_name_hosts_sequence():
    play = Play.load({'hosts': ['host1', 'host2']})

    assert play.get_name() == 'host1,host2'


def test_play_hosts_template_expression():
    play = Play.load({'hosts': "{{ target_hosts }}"})

    assert play.get_name() == '{{ target_hosts }}'


@pytest.mark.parametrize(
    'call',
    (
        '_load_tasks',
        '_load_pre_tasks',
        '_load_post_tasks',
        '_load_handlers',
        '_load_roles',
    )
)
def test_bad_blocks_roles(mocker, call):
    mocker.patch('ansible.playbook.play.load_list_of_blocks', side_effect=AssertionError('Raised intentionally'))
    mocker.patch('ansible.playbook.play.load_list_of_roles', side_effect=AssertionError('Raised intentionally'))

    play = Play.load({})
    with pytest.raises(AnsibleParserError, match='A malformed (block|(role declaration)) was encountered'):
        getattr(play, call)('', None)



# (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# most of it copied from AWX's scan_packages module

from __future__ import annotations


DOCUMENTATION = """
module: package_facts
short_description: Package information as facts
description:
  - Return information about installed packages as facts.
options:
  manager:
    description:
      - The package manager(s) used by the system so we can query the package information.
        This is a list and can support multiple package managers per system, since version 2.8.
      - The V(portage) and V(pkg) options were added in version 2.8.
      - The V(apk) option was added in version 2.11.
      - The V(pkg_info)' option was added in version 2.13.
      - Aliases were added in 2.18, to support using C(manager={{ansible_facts['pkg_mgr']}})
    default: ['auto']
    choices:
        auto: Depending on O(strategy), will match the first or all package managers provided, in order
        rpm: For RPM based distros, requires RPM Python bindings, not installed by default on Suse or Fedora 41+ (python3-rpm)
        yum: Alias to rpm
        dnf: Alias to rpm
        dnf5: Alias to rpm
        zypper: Alias to rpm
        apt: For DEB based distros, C(python-apt) package must be installed on targeted hosts
        portage: Handles ebuild packages, it requires the C(qlist) utility, which is part of 'app-portage/portage-utils'
        pkg: libpkg front end (FreeBSD)
        pkg5: Alias to pkg
        pkgng: Alias to pkg
        pacman: Archlinux package manager/builder
        apk: Alpine Linux package manager
        pkg_info: OpenBSD package manager
        openbsd_pkg: Alias to pkg_info
    type: list
    elements: str
  strategy:
    description:
      - This option controls how the module queries the package managers on the system.
    choices:
        first: returns only information for the first supported package manager available.
        all: returns information for all supported and available package managers on the system.
    default: 'first'
    type: str
    version_added: "2.8"
version_added: "2.5"
requirements:
    - See details per package manager in the O(manager) option.
author:
  - Matthew Jones (@matburt)
  - Brian Coca (@bcoca)
  - Adam Miller (@maxamillion)
extends_documentation_fragment:
  -  action_common_attributes
  -  action_common_attributes.facts
attributes:
    check_mode:
        support: full
    diff_mode:
        support: none
    facts:
        support: full
    platform:
        platforms: posix
"""

EXAMPLES = """
- name: Gather the package facts
  ansible.builtin.package_facts:
    manager: auto

- name: Print the package facts
  ansible.builtin.debug:
    var: ansible_facts.packages

- name: Check whether a package called foobar is installed
  ansible.builtin.debug:
    msg: "{{ ansible_facts.packages['foobar'] | length }} versions of foobar are installed!"
  when: "'foobar' in ansible_facts.packages"

"""

RETURN = """
ansible_facts:
  description: Facts to add to ansible_facts.
  returned: always
  type: complex
  contains:
    packages:
      description:
        - Maps the package name to a non-empty list of dicts with package information.
        - Every dict in the list corresponds to one installed version of the package.
        - The fields described below are present for all package managers. Depending on the
          package manager, there might be more fields for a package.
      returned: when operating system level package manager is specified or auto detected manager
      type: dict
      contains:
        name:
          description: The package's name.
          returned: always
          type: str
        version:
          description: The package's version.
          returned: always
          type: str
        source:
          description: Where information on the package came from.
          returned: always
          type: str
      sample: |-
        {
          "packages": {
            "kernel": [
              {
                "name": "kernel",
                "source": "rpm",
                "version": "3.10.0",
                ...
              },
              {
                "name": "kernel",
                "source": "rpm",
                "version": "3.10.0",
                ...
              },
              ...
            ],
            "kernel-tools": [
              {
                "name": "kernel-tools",
                "source": "rpm",
                "version": "3.10.0",
                ...
              }
            ],
            ...
          }
        }
        # Sample rpm
        {
          "packages": {
            "kernel": [
              {
                "arch": "x86_64",
                "epoch": null,
                "name": "kernel",
                "release": "514.26.2.el7",
                "source": "rpm",
                "version": "3.10.0"
              },
              {
                "arch": "x86_64",
                "epoch": null,
                "name": "kernel",
                "release": "514.16.1.el7",
                "source": "rpm",
                "version": "3.10.0"
              },
              {
                "arch": "x86_64",
                "epoch": null,
                "name": "kernel",
                "release": "514.10.2.el7",
                "source": "rpm",
                "version": "3.10.0"
              },
              {
                "arch": "x86_64",
                "epoch": null,
                "name": "kernel",
                "release": "514.21.1.el7",
                "source": "rpm",
                "version": "3.10.0"
              },
              {
                "arch": "x86_64",
                "epoch": null,
                "name": "kernel",
                "release": "693.2.2.el7",
                "source": "rpm",
                "version": "3.10.0"
              }
            ],
            "kernel-tools": [
              {
                "arch": "x86_64",
                "epoch": null,
                "name": "kernel-tools",
                "release": "693.2.2.el7",
                "source": "rpm",
                "version": "3.10.0"
              }
            ],
            "kernel-tools-libs": [
              {
                "arch": "x86_64",
                "epoch": null,
                "name": "kernel-tools-libs",
                "release": "693.2.2.el7",
                "source": "rpm",
                "version": "3.10.0"
              }
            ],
          }
        }
        # Sample deb
        {
          "packages": {
            "libbz2-1.0": [
              {
                "version": "1.0.6-5",
                "source": "apt",
                "arch": "amd64",
                "name": "libbz2-1.0"
              }
            ],
            "patch": [
              {
                "version": "2.7.1-4ubuntu1",
                "source": "apt",
                "arch": "amd64",
                "name": "patch"
              }
            ],
          }
        }
        # Sample pkg_info
        {
          "packages": {
            "curl": [
              {
                  "name": "curl",
                  "source": "pkg_info",
                  "version": "7.79.0"
              }
            ],
            "intel-firmware": [
              {
                  "name": "intel-firmware",
                  "source": "pkg_info",
                  "version": "20210608v0"
              }
            ],
          }
        }
"""

import re

from ansible.module_utils.common.text.converters import to_native, to_text
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.locale import get_best_parsable_locale
from ansible.module_utils.facts.packages import CLIMgr, RespawningLibMgr, get_all_pkg_managers


ALIASES = {
    'rpm': ['dnf', 'dnf5', 'yum' , 'zypper'],
    'pkg': ['pkg5', 'pkgng'],
    'pkg_info': ['openbsd_pkg'],
}


class RPM(RespawningLibMgr):

    LIB = 'rpm'
    CLI_BINARIES = ['rpm']
    INTERPRETERS = [
        '/usr/libexec/platform-python',
        '/usr/bin/python3',
    ]

    def list_installed(self):
        return self._lib.TransactionSet().dbMatch()

    def get_package_details(self, package):
        return dict(name=to_text(package[self._lib.RPMTAG_NAME]),
                    version=to_text(package[self._lib.RPMTAG_VERSION]),
                    release=to_text(package[self._lib.RPMTAG_RELEASE]),
                    epoch=to_text(package[self._lib.RPMTAG_EPOCH]),
                    arch=to_text(package[self._lib.RPMTAG_ARCH]),)


class APT(RespawningLibMgr):

    LIB = 'apt'
    CLI_BINARIES = ['apt', 'apt-get', 'aptitude']

    def __init__(self):
        self._cache = None
        super(APT, self).__init__()

    @property
    def pkg_cache(self):
        if self._cache is not None:
            return self._cache

        self._cache = self._lib.Cache()
        return self._cache

    def list_installed(self):
        # Store the cache to avoid running pkg_cache() for each item in the comprehension, which is very slow
        cache = self.pkg_cache
        return [pk for pk in cache.keys() if cache[pk].is_installed]

    def get_package_details(self, package):
        ac_pkg = self.pkg_cache[package].installed
        return dict(name=package, version=ac_pkg.version, arch=ac_pkg.architecture, category=ac_pkg.section, origin=ac_pkg.origins[0].origin)


class PACMAN(CLIMgr):

    CLI = 'pacman'

    def list_installed(self):
        locale = get_best_parsable_locale(module)
        rc, out, err = module.run_command([self._cli, '-Qi'], environ_update=dict(LC_ALL=locale))
        if rc != 0 or err:
            raise Exception("Unable to list packages rc=%s : %s" % (rc, err))
        return out.split("\n\n")[:-1]

    def get_package_details(self, package):
        # parse values of details that might extend over several lines
        raw_pkg_details = {}
        last_detail = None
        for line in package.splitlines():
            m = re.match(r"([\w ]*[\w]) +: (.*)", line)
            if m:
                last_detail = m.group(1)
                raw_pkg_details[last_detail] = m.group(2)
            else:
                # append value to previous detail
                raw_pkg_details[last_detail] = raw_pkg_details[last_detail] + "  " + line.lstrip()

        provides = None
        if raw_pkg_details['Provides'] != 'None':
            provides = [
                p.split('=')[0]
                for p in raw_pkg_details['Provides'].split('  ')
            ]

        return {
            'name': raw_pkg_details['Name'],
            'version': raw_pkg_details['Version'],
            'arch': raw_pkg_details['Architecture'],
            'provides': provides,
        }


class PKG(CLIMgr):

    CLI = 'pkg'
    atoms = ['name', 'version', 'origin', 'installed', 'automatic', 'arch', 'category', 'prefix', 'vital']

    def list_installed(self):
        rc, out, err = module.run_command([self._cli, 'query', "%%%s" % '\t%'.join(['n', 'v', 'R', 't', 'a', 'q', 'o', 'p', 'V'])])
        if rc != 0 or err:
            raise Exception("Unable to list packages rc=%s : %s" % (rc, err))
        return out.splitlines()

    def get_package_details(self, package):

        pkg = dict(zip(self.atoms, package.split('\t')))

        if 'arch' in pkg:
            try:
                pkg['arch'] = pkg['arch'].split(':')[2]
            except IndexError:
                pass

        if 'automatic' in pkg:
            pkg['automatic'] = bool(int(pkg['automatic']))

        if 'category' in pkg:
            pkg['category'] = pkg['category'].split('/', 1)[0]

        if 'version' in pkg:
            if ',' in pkg['version']:
                pkg['version'], pkg['port_epoch'] = pkg['version'].split(',', 1)
            else:
                pkg['port_epoch'] = 0

            if '_' in pkg['version']:
                pkg['version'], pkg['revision'] = pkg['version'].split('_', 1)
            else:
                pkg['revision'] = '0'

        if 'vital' in pkg:
            pkg['vital'] = bool(int(pkg['vital']))

        return pkg


class PORTAGE(CLIMgr):

    CLI = 'qlist'
    atoms = ['category', 'name', 'version', 'ebuild_revision', 'slots', 'prefixes', 'sufixes']

    def list_installed(self):
        rc, out, err = module.run_command(' '.join([self._cli, '-Iv', '|', 'xargs', '-n', '1024', 'qatom']), use_unsafe_shell=True)
        if rc != 0:
            raise RuntimeError("Unable to list packages rc=%s : %s" % (rc, to_native(err)))
        return out.splitlines()

    def get_package_details(self, package):
        return dict(zip(self.atoms, package.split()))


class APK(CLIMgr):

    CLI = 'apk'

    def list_installed(self):
        rc, out, err = module.run_command([self._cli, 'info', '-v'])
        if rc != 0:
            raise Exception("Unable to list packages rc=%s : %s" % (rc, err))
        return out.splitlines()

    def get_package_details(self, package):
        raw_pkg_details = {'name': package, 'version': '', 'release': ''}
        nvr = package.rsplit('-', 2)
        try:
            return {
                'name': nvr[0],
                'version': nvr[1],
                'release': nvr[2],
            }
        except IndexError:
            return raw_pkg_details


class PKG_INFO(CLIMgr):

    CLI = 'pkg_info'

    def list_installed(self):
        rc, out, err = module.run_command([self._cli, '-a'])
        if rc != 0 or err:
            raise Exception("Unable to list packages rc=%s : %s" % (rc, err))
        return out.splitlines()

    def get_package_details(self, package):
        raw_pkg_details = {'name': package, 'version': ''}
        details = package.split(maxsplit=1)[0].rsplit('-', maxsplit=1)

        try:
            return {
                'name': details[0],
                'version': details[1],
            }
        except IndexError:
            return raw_pkg_details


def main():

    # get supported pkg managers
    PKG_MANAGERS = get_all_pkg_managers()
    PKG_MANAGER_NAMES = sorted([x.lower() for x in PKG_MANAGERS.keys()])
    # add aliases
    PKG_MANAGER_NAMES.extend([alias for alist in ALIASES.values() for alias in alist])

    # start work
    global module

    # choices are not set for 'manager' as they are computed dynamically and validated below instead of in argspec
    module = AnsibleModule(argument_spec=dict(manager={'type': 'list', 'elements': 'str', 'default': ['auto']},
                                              strategy={'choices': ['first', 'all'], 'default': 'first'}),
                           supports_check_mode=True)
    packages = {}
    results = {'ansible_facts': {}}
    managers = [x.lower() for x in module.params['manager']]
    strategy = module.params['strategy']

    if 'auto' in managers:
        # keep order from user, we do dedupe below
        managers.extend(PKG_MANAGER_NAMES)
        managers.remove('auto')

    unsupported = set(managers).difference(PKG_MANAGER_NAMES)
    if unsupported:
        if 'auto' in module.params['manager']:
            msg = 'Could not auto detect a usable package manager, check warnings for details.'
        else:
            msg = 'Unsupported package managers requested: %s' % (', '.join(unsupported))
        module.fail_json(msg=msg)

    found = 0
    seen = set()
    for pkgmgr in managers:

        if strategy == 'first' and found:
            break

        # substitute aliases for aliased
        for aliased in ALIASES.keys():
            if pkgmgr in ALIASES[aliased]:
                pkgmgr = aliased
                break

        # dedupe as per above
        if pkgmgr in seen:
            continue

        seen.add(pkgmgr)

        manager = PKG_MANAGERS[pkgmgr]()
        try:
            packages_found = {}
            if manager.is_available(handle_exceptions=False):
                try:
                    packages_found = manager.get_packages()
                except Exception as e:
                    module.warn('Failed to retrieve packages with %s: %s' % (pkgmgr, to_text(e)))

            # only consider 'found' if it results in something
            if packages_found:
                found += 1
                for k in packages_found.keys():
                    if k in packages:
                        packages[k].extend(packages_found[k])
                    else:
                        packages[k] = packages_found[k]
            else:
                module.warn('Found "%s" but no associated packages' % (pkgmgr))

        except Exception as e:
            if pkgmgr in module.params['manager']:
                module.warn('Requested package manager %s was not usable by this module: %s' % (pkgmgr, to_text(e)))

    if found == 0:
        msg = ('Could not detect a supported package manager from the following list: %s, '
               'or the required Python library is not installed. Check warnings for details.' % managers)
        module.fail_json(msg=msg)

    # Set the facts, this will override the facts in ansible_facts that might exist from previous runs
    # when using operating system level or distribution package managers
    results['ansible_facts']['packages'] = packages

    module.exit_json(**results)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations


class ModuleDocFragment(object):
    DOCUMENTATION = r"""
options: {}
version_added: 1.0.0
"""



from __future__ import annotations

import contextlib
import signal
import time
import typing as t

import pytest

from ansible._internal._errors import _alarm_timeout
from ansible._internal._errors._alarm_timeout import AnsibleTimeoutError

pytestmark = pytest.mark.usefixtures("assert_sigalrm_state")


@pytest.fixture
def assert_sigalrm_state() -> t.Iterator[None]:
    """Fixture to ensure that SIGALRM state is as-expected before and after each test."""
    assert signal.alarm(0) == 0  # disable alarm before resetting the default handler
    assert signal.signal(signal.SIGALRM, signal.SIG_DFL) == signal.SIG_DFL

    try:
        yield
    finally:
        assert signal.alarm(0) == 0
        assert signal.signal(signal.SIGALRM, signal.SIG_DFL) == signal.SIG_DFL


@pytest.mark.parametrize("timeout", (0, 1, None))
def test_alarm_timeout_success(timeout: int | None) -> None:
    """Validate a non-timeout success scenario."""
    ran = False

    with _alarm_timeout.AnsibleTimeoutError.alarm_timeout(timeout):
        time.sleep(0.01)
        ran = True

    assert ran


def test_alarm_timeout_timeout() -> None:
    """Validate a happy-path timeout scenario."""
    ran = False
    timeout_sec = 1

    with pytest.raises(AnsibleTimeoutError) as error:
        with _alarm_timeout.AnsibleTimeoutError.alarm_timeout(timeout_sec):
            time.sleep(timeout_sec + 1)
            ran = True  # pragma: nocover

    assert not ran
    assert error.value.timeout == timeout_sec


@pytest.mark.parametrize("timeout,expected_error_type,expected_error_pattern", (
    (-1, ValueError, "Timeout.*invalid.*between"),
    (100_000_001, ValueError, "Timeout.*invalid.*between"),
    (0.1, TypeError, "requires 'int' argument.*'float'"),
    ("1", TypeError, "requires 'int' argument.*'str'"),
))
def test_alarm_timeout_bad_values(timeout: t.Any, expected_error_type: type[Exception], expected_error_pattern: str) -> None:
    """Validate behavior for invalid inputs."""
    ran = False

    with pytest.raises(expected_error_type, match=expected_error_pattern):
        with _alarm_timeout.AnsibleTimeoutError.alarm_timeout(timeout):
            ran = True  # pragma: nocover

    assert not ran


def test_alarm_timeout_bad_state() -> None:
    """Validate alarm state error handling."""
    def call_it():
        ran = False

        with pytest.raises(RuntimeError, match="existing alarm"):
            with _alarm_timeout.AnsibleTimeoutError.alarm_timeout(1):
                ran = True  # pragma: nocover

        assert not ran

    try:
        # non-default SIGALRM handler present
        signal.signal(signal.SIGALRM, lambda _s, _f: None)
        call_it()
    finally:
        signal.signal(signal.SIGALRM, signal.SIG_DFL)

    try:
        # alarm already set
        signal.alarm(10000)
        call_it()
    finally:
        signal.signal(signal.SIGALRM, signal.SIG_DFL)

    ran_outer = ran_inner = False

    # nested alarm_timeouts
    with pytest.raises(RuntimeError, match="existing alarm"):
        with _alarm_timeout.AnsibleTimeoutError.alarm_timeout(1):
            ran_outer = True

            with _alarm_timeout.AnsibleTimeoutError.alarm_timeout(1):
                ran_inner = True  # pragma: nocover

    assert not ran_inner
    assert ran_outer


def test_alarm_timeout_raise():
    """Ensure that an exception raised in the wrapped scope propagates correctly."""
    with pytest.raises(NotImplementedError):
        with _alarm_timeout.AnsibleTimeoutError.alarm_timeout(1):
            raise NotImplementedError()


def test_alarm_timeout_escape_broad_exception():
    """Ensure that the timeout exception can escape a broad exception handler in the wrapped scope."""
    with pytest.raises(AnsibleTimeoutError):
        with _alarm_timeout.AnsibleTimeoutError.alarm_timeout(1):
            with contextlib.suppress(Exception):
                time.sleep(3)



# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import annotations

import json
import multiprocessing.resource_tracker
import os
import re
import sys
import typing as t

from multiprocessing.shared_memory import SharedMemory


def main() -> t.Never:
    if len(sys.argv) > 1:
        exit_code = 0 if handle_prompt(sys.argv[1]) else 1
    else:
        exit_code = 1

    sys.exit(exit_code)


def handle_prompt(prompt: str) -> bool:
    if re.search(r'(The authenticity of host |differs from the key for the IP address)', prompt):
        sys.stdout.write('no')
        sys.stdout.flush()
        return True

    # deprecated: description='Python 3.13 and later support track' python_version='3.12'
    can_track = sys.version_info[:2] >= (3, 13)
    kwargs = dict(track=False) if can_track else {}

    # This SharedMemory instance is intentionally not closed or unlinked.
    # Closing will occur naturally in the SharedMemory finalizer.
    # Unlinking is the responsibility of the process which created it.
    shm = SharedMemory(name=os.environ['_ANSIBLE_SSH_ASKPASS_SHM'], **kwargs)

    if not can_track:
        # When track=False is not available, we must unregister explicitly, since it otherwise only occurs during unlink.
        # This avoids resource tracker noise on stderr during process exit.
        multiprocessing.resource_tracker.unregister(shm._name, 'shared_memory')

    cfg = json.loads(shm.buf.tobytes().rstrip(b'\x00'))

    if cfg['prompt'] not in prompt:
        return False

    # Report the password provided by the SharedMemory instance.
    # The contents are left untouched after consumption to allow subsequent attempts to succeed.
    # This can occur when multiple password prompting methods are enabled, such as password and keyboard-interactive, which is the default on macOS.
    sys.stdout.write(cfg['password'])
    sys.stdout.flush()
    return True



# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

from ansible_collections.testns.testcol.plugins.module_utils import Display
# Test for https://github.com/ansible/ansible/issues/85754
from ...module_utils import Display

display = Display()


def nochange(a):
    return a


class FilterModule(object):
    """ Ansible core jinja2 filters """

    def filters(self):
        return {
            'noop': nochange,
            'nested': nochange,
        }



# Copyright (c) 2024 Ansible Project
# Simplified BSD License (see licenses/simplified_bsd.txt or https://opensource.org/licenses/BSD-2-Clause)

"""Internal utility code for supporting traceback reporting."""

from __future__ import annotations

import enum
import traceback

from . import _stack


class TracebackEvent(enum.Enum):
    """The events for which tracebacks can be enabled."""

    ERROR = enum.auto()
    WARNING = enum.auto()
    DEPRECATED = enum.auto()
    DEPRECATED_VALUE = enum.auto()  # implies DEPRECATED


def traceback_for() -> list[str]:
    """Return a list of traceback event names (not enums) which are enabled."""
    return [value.name.lower() for value in TracebackEvent if is_traceback_enabled(value)]


def is_traceback_enabled(event: TracebackEvent) -> bool:
    """Return True if tracebacks are enabled for the specified event, otherwise return False."""
    return _is_traceback_enabled(event)


def maybe_capture_traceback(msg: str, event: TracebackEvent) -> str | None:
    """
    Optionally capture a traceback for the current call stack, formatted as a string, if the specified traceback event is enabled.
    Frames marked with the `_skip_stackwalk` local are omitted.
    """
    _skip_stackwalk = True

    if not is_traceback_enabled(event):
        return None

    tb_lines = []

    if frame_info := _stack.caller_frame():
        # DTFIX-FUTURE: rewrite target-side tracebacks to point at controller-side paths?
        tb_lines.append('Traceback (most recent call last):\n')
        tb_lines.extend(traceback.format_stack(frame_info.frame))
        tb_lines.append(f'Message: {msg}\n')
    else:
        tb_lines.append('(frame not found)\n')  # pragma: nocover

    return ''.join(tb_lines)


def maybe_extract_traceback(exception: BaseException, event: TracebackEvent) -> str | None:
    """Optionally extract a formatted traceback from the given exception, if the specified traceback event is enabled."""

    if not is_traceback_enabled(event):
        return None

    # deprecated: description='use the single-arg version of format_traceback' python_version='3.9'
    tb_lines = traceback.format_exception(type(exception), exception, exception.__traceback__)

    return ''.join(tb_lines)


_module_tracebacks_enabled_events: frozenset[TracebackEvent] | None = None
"""Cached enabled TracebackEvent values extracted from `_ansible_tracebacks_for` module arg."""


def _is_module_traceback_enabled(event: TracebackEvent) -> bool:
    """Module utility function to lazily load traceback config and determine if traceback collection is enabled for the specified event."""
    global _module_tracebacks_enabled_events

    if _module_tracebacks_enabled_events is None:
        try:
            # Suboptimal error handling, but since import order can matter, and this is a critical error path, better to fail silently
            # than to mask the triggering error by issuing a new error/warning here.
            from ..basic import _PARSED_MODULE_ARGS

            _module_tracebacks_enabled_events = frozenset(
                TracebackEvent[value.upper()] for value in _PARSED_MODULE_ARGS.get('_ansible_tracebacks_for', [])
            )  # type: ignore[union-attr]
        except BaseException:
            return True  # if things failed early enough that we can't figure this out, assume we want a traceback for troubleshooting

    return event in _module_tracebacks_enabled_events


_is_traceback_enabled = _is_module_traceback_enabled
"""Callable to determine if tracebacks are enabled. Overridden on the controller by display. Use `is_traceback_enabled` instead of calling this directly."""



# (c) 2017 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import annotations

DOCUMENTATION = """
    name: vars
    author: Ansible Core Team
    version_added: "2.5"
    short_description: Lookup templated value of variables
    description:
      - 'Retrieves the value of an Ansible variable. Note: Only returns top level variable names.'
    options:
      _terms:
        description: The variable names to look up.
        required: True
      default:
        description:
            - What to return if a variable is undefined.
            - If no default is set, it will result in an error if any of the variables is undefined.
    seealso:
    - plugin_type: lookup
      plugin: ansible.builtin.varnames

"""

EXAMPLES = """
- name: Show value of 'variablename'
  ansible.builtin.debug: msg="{{ lookup('ansible.builtin.vars', 'variabl' + myvar) }}"
  vars:
    variablename: hello
    myvar: ename

- name: Show default empty since i dont have 'variablnotename'
  ansible.builtin.debug: msg="{{ lookup('ansible.builtin.vars', 'variabl' + myvar, default='') }}"
  vars:
    variablename: hello
    myvar: notename

- name: Produce an error since i dont have 'variablnotename'
  ansible.builtin.debug: msg="{{ q('vars', 'variabl' + myvar) }}"
  ignore_errors: True
  vars:
    variablename: hello
    myvar: notename

- name: find several related variables
  ansible.builtin.debug: msg="{{ query('ansible.builtin.vars', 'ansible_play_hosts', 'ansible_play_batch', 'ansible_play_hosts_all') }}"

- name: show values from variables found via varnames (note "*" is used to dereference the list to a 'list of arguments')
  debug: msg="{{ q('vars', *q('varnames', 'ansible_play_.+')) }}"

- name: Access nested variables
  ansible.builtin.debug: msg="{{ lookup('ansible.builtin.vars', 'variabl' + myvar).sub_var }}"
  ignore_errors: True
  vars:
    variablename:
        sub_var: 12
    myvar: ename

- name: alternate way to find some 'prefixed vars' in loop
  ansible.builtin.debug: msg="{{ lookup('ansible.builtin.vars', 'ansible_play_' + item) }}"
  loop:
    - hosts
    - batch
    - hosts_all
"""

RETURN = """
_value:
  description:
    - value of the variables requested.
  type: list
  elements: raw
"""

from ansible.errors import AnsibleTypeError
from ansible.plugins.lookup import LookupBase
from ansible.module_utils.datatag import native_type_name
from ansible._internal._templating import _jinja_bits


class LookupModule(LookupBase):
    def run(self, terms, variables, **kwargs):
        self.set_options(var_options=variables, direct=kwargs)

        default = self.get_option('default')

        ret = []

        for term in terms:
            if not isinstance(term, str):
                raise AnsibleTypeError(f'Variable name must be {native_type_name(str)!r} not {native_type_name(term)!r}.', obj=term)

            try:
                value = variables[term]
            except KeyError:
                if default is None:
                    value = _jinja_bits._undef(f'No variable named {term!r} was found.')
                else:
                    value = default

            ret.append(value)

        return self._templar._engine.template(ret)



from __future__ import annotations as _annotations


def concat_message(left: str, right: str) -> str:
    """Normalize `left` by removing trailing punctuation and spaces before appending new punctuation and `right`."""
    return f'{left.rstrip(". ")}: {right}'



# -*- coding: utf-8 -*-
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations


DOCUMENTATION = r"""
---
author: Ansible Core Team (@ansible)
module: import_role
short_description: Import a role into a play
description:
  - Much like the C(roles:) keyword, this task loads a role, but it allows you to control when the role tasks run in
    between other tasks of the play.
  - Most keywords, loops and conditionals will only be applied to the imported tasks, not to this statement itself. If
    you want the opposite behavior, use M(ansible.builtin.include_role) instead.
  - Does not work in handlers.
version_added: '2.4'
options:
  name:
    description:
      - The name of the role to be executed.
    type: str
    required: true
  tasks_from:
    description:
      - File to load from a role's C(tasks/) directory.
    type: str
    default: main
  vars_from:
    description:
      - File to load from a role's C(vars/) directory.
    type: str
    default: main
  defaults_from:
    description:
      - File to load from a role's C(defaults/) directory.
    type: str
    default: main
  allow_duplicates:
    description:
      - Overrides the role's metadata setting to allow using a role more than once with the same parameters.
    type: bool
    default: yes
  handlers_from:
    description:
      - File to load from a role's C(handlers/) directory.
    type: str
    default: main
    version_added: '2.8'
  rolespec_validate:
    description:
      - Perform role argument spec validation if an argument spec is defined.
    type: bool
    default: yes
    version_added: '2.11'
  public:
    description:
      - This option dictates whether the role's C(vars) and C(defaults) are exposed to the play.
      - Variables are exposed to the play at playbook parsing time, and available to earlier roles and tasks as well unlike C(include_role).
      - The default depends on the configuration option R(DEFAULT_PRIVATE_ROLE_VARS, DEFAULT_PRIVATE_ROLE_VARS).
    type: bool
    default: yes
    version_added: '2.17'
extends_documentation_fragment:
    - action_common_attributes
    - action_common_attributes.conn
    - action_common_attributes.flow
    - action_core
    - action_core.import
attributes:
    check_mode:
      support: full
    diff_mode:
      support: none
notes:
  - Handlers are made available to the whole play.
  - Since Ansible 2.7 variables defined in C(vars) and C(defaults) for the role are exposed to the play at playbook parsing time.
    Due to this, these variables will be accessible to roles and tasks executed before the location of the
    M(ansible.builtin.import_role) task.
  - Unlike M(ansible.builtin.include_role) variable exposure is not configurable, and will always be exposed.
seealso:
- module: ansible.builtin.import_playbook
- module: ansible.builtin.import_tasks
- module: ansible.builtin.include_role
- module: ansible.builtin.include_tasks
- ref: playbooks_reuse
  description: More information related to including and importing playbooks, roles and tasks.
"""

EXAMPLES = r"""
- hosts: all
  tasks:
    - ansible.builtin.import_role:
        name: myrole

    - name: Run tasks/other.yaml instead of 'main'
      ansible.builtin.import_role:
        name: myrole
        tasks_from: other

    - name: Pass variables to role
      ansible.builtin.import_role:
        name: myrole
      vars:
        rolevar1: value from task

    - name: Apply condition to each task in role
      ansible.builtin.import_role:
        name: myrole
      when: not idontwanttorun
"""

RETURN = r"""
# This module does not return anything except tasks to execute.
"""
